@@ -31,10 +31,12 @@ script="$0"
 scriptdir=$(dirname "$script")
 script=$(basename "$script")
 
-. "$scriptdir/functions.sh"
+. "$scriptdir/scripts/functions.sh"
 
 cd "$scriptdir"
 
+# Simply prints the help message and quits based on the argument.
+# @param val  The value to pass to exit. Must be an integer.
 usage() {
 
 	if [ $# -gt 0 ]; then
@@ -52,16 +54,16 @@ usage() {
 	printf '    %s --help\n' "$script"
 	printf '    %s [-a|-bD|-dB|-c] [-CEfgGHlmMNPtTvz] [-O OPT_LEVEL] [-k KARATSUBA_LEN]\n' "$script"
 	printf '    %s \\\n' "$script"
-	printf '       [--library|--bc-only --disable-dc|--dc-only --disable-bc|--coverage]\\\n'
-	printf '       [--force --debug --disable-extra-math --disable-generated-tests]    \\\n'
-	printf '       [--disable-history --disable-man-pages --disable-nls]               \\\n'
-	printf '       [--disable-prompt --disable-strip] [--install-all-locales]          \\\n'
-	printf '       [--opt=OPT_LEVEL] [--karatsuba-len=KARATSUBA_LEN]                   \\\n'
-	printf '       [--prefix=PREFIX] [--bindir=BINDIR] [--datarootdir=DATAROOTDIR]     \\\n'
-	printf '       [--datadir=DATADIR] [--mandir=MANDIR] [--man1dir=MAN1DIR]           \\\n'
+	printf '       [--library|--bc-only --disable-dc|--dc-only --disable-bc|--coverage]  \\\n'
+	printf '       [--force --debug --disable-extra-math --disable-generated-tests]      \\\n'
+	printf '       [--disable-history --disable-man-pages --disable-nls --disable-strip] \\\n'
+	printf '       [--install-all-locales] [--opt=OPT_LEVEL]                             \\\n'
+	printf '       [--karatsuba-len=KARATSUBA_LEN]                                       \\\n'
+	printf '       [--prefix=PREFIX] [--bindir=BINDIR] [--datarootdir=DATAROOTDIR]       \\\n'
+	printf '       [--datadir=DATADIR] [--mandir=MANDIR] [--man1dir=MAN1DIR]             \\\n'
 	printf '\n'
 	printf '    -a, --library\n'
-	printf '        Build the libbc instead of the programs. This is meant to be used with\n'
+	printf '        Build the libbcl instead of the programs. This is meant to be used with\n'
 	printf '        Other software like programming languages that want to make use of the\n'
 	printf '        parsing and math capabilities. This option will install headers using\n'
 	printf '        `make install`.\n'
@@ -72,7 +74,7 @@ usage() {
 	printf '        Disable bc. It is an error if "-b", "--bc-only", "-D", or "--disable-dc"\n'
 	printf '        are specified too.\n'
 	printf '    -c, --coverage\n'
-	printf '        Generate test coverage code. Requires gcov and regcovr.\n'
+	printf '        Generate test coverage code. Requires gcov and gcovr.\n'
 	printf '        It is an error if either "-b" ("-D") or "-d" ("-B") is specified.\n'
 	printf '        Requires a compiler that use gcc-compatible coverage options\n'
 	printf '    -C, --disable-clean\n'
@@ -121,10 +123,14 @@ usage() {
 	printf '        Set the optimization level. This can also be included in the CFLAGS,\n'
 	printf '        but it is provided, so maintainers can build optimized debug builds.\n'
 	printf '        This is passed through to the compiler, so it must be supported.\n'
-	printf '    -P, --disable-prompt\n'
-	printf '        Disables the prompt in the built bc. The prompt will never show up,\n'
-	printf '        or in other words, it will be permanently disabled and cannot be\n'
-	printf '        enabled.\n'
+	printf '    -s SETTING, --set-default-on SETTING\n'
+	printf '        Set the default named by SETTING to on. See below for possible values\n'
+	printf '        for SETTING. For multiple instances of the -s or -S for the the same\n'
+	printf '        setting, the last one is used.\n'
+	printf '    -S SETTING, --set-default-off SETTING\n'
+	printf '        Set the default named by SETTING to off. See below for possible values\n'
+	printf '        for SETTING. For multiple instances of the -s or -S for the the same\n'
+	printf '        setting, the last one is used.\n'
 	printf '    -t, --enable-test-timing\n'
 	printf '        Enable the timing of tests. This is for development only.\n'
 	printf '    -T, --disable-strip\n'
@@ -239,10 +245,66 @@ usage() {
 	printf '\n'
 	printf 'WARNING: even though `configure.sh` supports both option types, short and\n'
 	printf 'long, it does not support handling both at the same time. Use only one type.\n'
+	printf '\n'
+	printf 'Settings\n'
+	printf '========\n'
+	printf '\n'
+	printf 'bc and dc have some settings that, while they cannot be removed by build time\n'
+	printf 'options, can have their defaults changed at build time by packagers. Users are\n'
+	printf 'also able to change each setting with environment variables.\n'
+	printf '\n'
+	printf 'The following is a table of settings, along with their default values and the\n'
+	printf 'environment variables users can use to change them. (For the defaults, non-zero\n'
+	printf 'means on, and zero means off.)\n'
+	printf '\n'
+	printf '| Setting         | Description          | Default      | Env Variable         |\n'
+	printf '| =============== | ==================== | ============ | ==================== |\n'
+	printf '| bc.banner       | Whether to display   |            0 | BC_BANNER            |\n'
+	printf '|                 | the bc version       |              |                      |\n'
+	printf '|                 | banner when in       |              |                      |\n'
+	printf '|                 | interactive mode.    |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| bc.sigint_reset | Whether SIGINT will  |            1 | BC_SIGINT_RESET      |\n'
+	printf '|                 | reset bc, instead of |              |                      |\n'
+	printf '|                 | exiting, when in     |              |                      |\n'
+	printf '|                 | interactive mode.    |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| dc.sigint_reset | Whether SIGINT will  |            1 | DC_SIGINT_RESET      |\n'
+	printf '|                 | reset dc, instead of |              |                      |\n'
+	printf '|                 | exiting, when in     |              |                      |\n'
+	printf '|                 | interactive mode.    |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| bc.tty_mode     | Whether TTY mode for |            1 | BC_TTY_MODE          |\n'
+	printf '|                 | bc should be on when |              |                      |\n'
+	printf '|                 | available.           |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| dc.tty_mode     | Whether TTY mode for |            0 | BC_TTY_MODE          |\n'
+	printf '|                 | dc should be on when |              |                      |\n'
+	printf '|                 | available.           |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| bc.prompt       | Whether the prompt   | $BC_TTY_MODE | BC_PROMPT            |\n'
+	printf '|                 | for bc should be on  |              |                      |\n'
+	printf '|                 | in tty mode.         |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '| dc.prompt       | Whether the prompt   | $DC_TTY_MODE | DC_PROMPT            |\n'
+	printf '|                 | for dc should be on  |              |                      |\n'
+	printf '|                 | in tty mode.         |              |                      |\n'
+	printf '| --------------- | -------------------- | ------------ | -------------------- |\n'
+	printf '\n'
+	printf 'These settings are not meant to be changed on a whim. They are meant to ensure\n'
+	printf 'that this bc and dc will conform to the expectations of the user on each\n'
+	printf 'platform.\n'
 
 	exit "$_usage_val"
 }
 
+# Replaces a file extension in a filename. This is used mostly to turn filenames
+# like `src/num.c` into `src/num.o`. In other words, it helps to link targets to
+# the files they depend on.
+#
+# @param file  The filename.
+# @param ext1  The extension to replace.
+# @param ext2  The new extension.
 replace_ext() {
 
 	if [ "$#" -ne 3 ]; then
@@ -258,6 +320,13 @@ replace_ext() {
 	printf '%s\n' "$_replace_ext_result"
 }
 
+# Replaces a file extension in every filename given in a list. The list is just
+# a space-separated list of words, so filenames are expected to *not* have
+# spaces in them. See the documentation for `replace_ext()`.
+#
+# @param files  The list of space-separated filenames to replace extensions for.
+# @param ext1   The extension to replace.
+# @param ext2   The new extension.
 replace_exts() {
 
 	if [ "$#" -ne 3 ]; then
@@ -276,6 +345,17 @@ replace_exts() {
 	printf '%s\n' "$_replace_exts_result"
 }
 
+# Finds a placeholder in @a str and replaces it. This is the workhorse of
+# configure.sh. It's what replaces placeholders in Makefile.in with the data
+# needed for the chosen build. Below, you will see a lot of calls to this
+# function.
+#
+# Note that needle can never contain an exclamation point. For more information,
+# see substring_replace() in scripts/functions.sh.
+#
+# @param str          The string to find and replace placeholders in.
+# @param needle       The placeholder name.
+# @param replacement  The string to use to replace the placeholder.
 replace() {
 
 	if [ "$#" -ne 3 ]; then
@@ -289,6 +369,9 @@ replace() {
 	substring_replace "$_replace_str" "%%$_replace_needle%%" "$_replace_replacement"
 }
 
+# This function finds all the source files that need to be built. If there is
+# only one argument and it is empty, then all source files are built. Otherwise,
+# the arguments are all assumed to be source files that should *not* be built.
 find_src_files() {
 
 	if [ "$#" -ge 1 ] && [ "$1" != "" ]; then
@@ -306,6 +389,11 @@ find_src_files() {
 	printf '%s\n' $(find src/ -depth -name "*.c" $_find_src_files_args)
 }
 
+# This function generates a list of files to go into the Makefile. It generates
+# the list of object files, as well as the list of test coverage files.
+#
+# @param contents  The contents of the Makefile template to put the list of
+#                  files into.
 gen_file_list() {
 
 	if [ "$#" -lt 1 ]; then
@@ -351,21 +439,25 @@ gen_file_list() {
 	printf '%s\n' "$_gen_file_list_contents"
 }
 
+# Generates the proper test targets for each test to have its own target. This
+# allows `make test` to run in parallel.
+#
+# @param name        Which calculator to generate tests for.
+# @param extra_math  An integer that, if non-zero, activates extra math tests.
+# @param time_tests  An integer that, if non-zero, tells the test suite to time
+#                    the execution of each test.
 gen_tests() {
 
 	_gen_tests_name="$1"
 	shift
 
-	_gen_tests_uname="$1"
-	shift
-
 	_gen_tests_extra_math="$1"
 	shift
 
 	_gen_tests_time_tests="$1"
 	shift
 
-	_gen_tests_extra_required=$(cat tests/extra_required.txt)
+	_gen_tests_extra_required=$(cat "$scriptdir/tests/extra_required.txt")
 
 	for _gen_tests_t in $(cat "$scriptdir/tests/$_gen_tests_name/all.txt"); do
 
@@ -388,6 +480,10 @@ gen_tests() {
 	done
 }
 
+# Generates a list of test targets that will be used as prerequisites for other
+# targets.
+#
+# @param name  The name of the calculator to generate test targets for.
 gen_test_targets() {
 
 	_gen_test_targets_name="$1"
@@ -402,6 +498,14 @@ gen_test_targets() {
 	printf '\n'
 }
 
+# Generates the proper script test targets for each script test to have its own
+# target. This allows `make test` to run in parallel.
+#
+# @param name        Which calculator to generate tests for.
+# @param extra_math  An integer that, if non-zero, activates extra math tests.
+# @param generate    An integer that, if non-zero, activates generated tests.
+# @param time_tests  An integer that, if non-zero, tells the test suite to time
+#                    the execution of each test.
 gen_script_tests() {
 
 	_gen_script_tests_name="$1"
@@ -429,6 +533,36 @@ gen_script_tests() {
 	done
 }
 
+set_default() {
+
+	_set_default_on="$1"
+	shift
+
+	_set_default_name="$1"
+	shift
+
+	# The reason that the variables that are being set do not have the same
+	# non-collision avoidance that the other variables do is that we *do* want
+	# the settings of these variables to leak out of the function. They adjust
+	# the settings outside of the function.
+	case "$_set_default_name" in
+
+		bc.banner) bc_default_banner="$_set_default_on" ;;
+		bc.sigint_reset) bc_default_sigint_reset="$_set_default_on" ;;
+		dc.sigint_reset) dc_default_sigint_reset="$_set_default_on" ;;
+		bc.tty_mode) bc_default_tty_mode="$_set_default_on" ;;
+		dc.tty_mode) dc_default_tty_mode="$_set_default_on" ;;
+		bc.prompt) bc_default_prompt="$_set_default_on" ;;
+		dc.prompt) dc_default_prompt="$_set_default_on" ;;
+		?) usage "Invalid setting: $_set_default_name" ;;
+
+	esac
+}
+
+# Generates a list of script test targets that will be used as prerequisites for
+# other targets.
+#
+# @param name  The name of the calculator to generate script test targets for.
 gen_script_test_targets() {
 
 	_gen_script_test_targets_name="$1"
@@ -446,6 +580,12 @@ gen_script_test_targets() {
 	printf '\n'
 }
 
+# This is a list of defaults, but it is also the list of possible options for
+# users to change.
+#
+# The development options are: force (force options even if they fail), valgrind
+# (build in a way suitable for valgrind testing), memcheck (same as valgrind),
+# and fuzzing (build in a way suitable for fuzzing).
 bc_only=0
 dc_only=0
 coverage=0
@@ -457,7 +597,6 @@ optimization=""
 generate_tests=1
 install_manpages=1
 nls=1
-prompt=1
 force=0
 strip_bin=1
 all_locales=0
@@ -468,7 +607,20 @@ vg=0
 memcheck=0
 clean=1
 
-while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
+# The empty strings are because they depend on TTY mode. If they are directly
+# set, though, they will be integers. We test for empty strings later.
+bc_default_banner=0
+bc_default_sigint_reset=1
+dc_default_sigint_reset=1
+bc_default_tty_mode=1
+dc_default_tty_mode=0
+bc_default_prompt=""
+dc_default_prompt=""
+
+# getopts is a POSIX utility, but it cannot handle long options. Thus, the
+# handling of long options is done by hand, and that's the reason that short and
+# long options cannot be mixed.
+while getopts "abBcdDEfgGhHk:lMmNO:S:s:tTvz-" opt; do
 
 	case "$opt" in
 		a) library=1 ;;
@@ -490,7 +642,8 @@ while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
 		M) install_manpages=0 ;;
 		N) nls=0 ;;
 		O) optimization="$OPTARG" ;;
-		P) prompt=0 ;;
+		S) set_default 0 "$OPTARG" ;;
+		s) set_default 1 "$OPTARG" ;;
 		t) time_tests=1 ;;
 		T) strip_bin=0 ;;
 		v) vg=1 ;;
@@ -591,6 +744,20 @@ while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
 					fi
 					optimization="$1"
 					shift ;;
+				set-default-on=?*) set_default 1 "$LONG_OPTARG" ;;
+				set-default-on)
+					if [ "$#" -lt 2 ]; then
+						usage "No argument given for '--$arg' option"
+					fi
+					set_default 1 "$1"
+					shift ;;
+				set-default-off=?*) set_default 0 "$LONG_OPTARG" ;;
+				set-default-off)
+					if [ "$#" -lt 2 ]; then
+						usage "No argument given for '--$arg' option"
+					fi
+					set_default 0 "$1"
+					shift ;;
 				disable-bc) dc_only=1 ;;
 				disable-dc) bc_only=1 ;;
 				disable-clean) clean=0 ;;
@@ -599,7 +766,6 @@ while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
 				disable-history) hist=0 ;;
 				disable-man-pages) install_manpages=0 ;;
 				disable-nls) nls=0 ;;
-				disable-prompt) prompt=0 ;;
 				disable-strip) strip_bin=0 ;;
 				enable-test-timing) time_tests=1 ;;
 				enable-valgrind) vg=1 ;;
@@ -625,27 +791,33 @@ while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
 			esac
 			shift
 			OPTIND=1 ;;
-		?) usage "Invalid option $opt" ;;
+		?) usage "Invalid option: $opt" ;;
 	esac
 
 done
 
+# Sometimes, developers don't want configure.sh to do a config clean. But
+# sometimes they do.
 if [ "$clean" -ne 0 ]; then
 	if [ -f ./Makefile ]; then
 		make clean_config > /dev/null
 	fi
 fi
 
+# It is an error to say that bc only should be built and likewise for dc.
 if [ "$bc_only" -eq 1 ] && [ "$dc_only" -eq 1 ]; then
 	usage "Can only specify one of -b(-D) or -d(-B)"
 fi
 
+# The library is mutually exclusive to the calculators, so it's an error to
+# give an option for either of them.
 if [ "$library" -ne 0 ]; then
 	if [ "$bc_only" -eq 1 ] || [ "$dc_only" -eq 1 ]; then
 		usage "Must not specify -b(-D) or -d(-B) when building the library"
 	fi
 fi
 
+# KARATSUBA_LEN must be an integer and must be 16 or greater.
 case $karatsuba_len in
 	(*[!0-9]*|'') usage "KARATSUBA_LEN is not a number" ;;
 	(*) ;;
@@ -668,6 +840,11 @@ fi
 if [ -z "$CC" ]; then
 	CC="c99"
 else
+
+	# I had users complain that, if they gave CFLAGS as part of CC, which
+	# autotools allows in its braindead way, the build would fail with an error.
+	# I don't like adjusting for autotools, but oh well. These lines puts the
+	# stuff after the first space into CFLAGS.
 	ccbase=$(basename "$CC")
 	suffix=" *"
 	prefix="* "
@@ -693,6 +870,8 @@ elif [ -z "$HOSTCC" ]; then
 fi
 
 if [ "$HOSTCC" != "$CC" ]; then
+
+	# Like above, this splits HOSTCC and HOSTCFLAGS.
 	ccbase=$(basename "$HOSTCC")
 	suffix=" *"
 	prefix="* "
@@ -717,14 +896,18 @@ elif [ -z "${HOSTCFLAGS+set}" ]; then
 	HOSTCFLAGS="$HOST_CFLAGS"
 fi
 
+# Store these for the cross compilation detection later.
+OLDCFLAGS="$CFLAGS"
+OLDHOSTCFLAGS="$HOSTCFLAGS"
+
 link="@printf 'No link necessary\\\\n'"
 main_exec="BC"
 executable="BC_EXEC"
 
-tests="test_bc timeconst test_dc"
+tests="test_bc timeconst test_dc test_history"
 
-bc_test="@tests/all.sh bc $extra_math 1 $generate_tests 0 \$(BC_EXEC)"
-dc_test="@tests/all.sh dc $extra_math 1 $generate_tests 0 \$(DC_EXEC)"
+bc_test="@tests/all.sh bc $extra_math 1 $generate_tests $time_tests \$(BC_EXEC)"
+dc_test="@tests/all.sh dc $extra_math 1 $generate_tests $time_tests \$(DC_EXEC)"
 
 timeconst="@tests/bc/timeconst.sh tests/bc/scripts/timeconst.bc \$(BC_EXEC)"
 
@@ -739,6 +922,9 @@ else
 	dc_test_exec='$(DC_EXEC)'
 fi
 
+test_bc_history_prereqs="test_bc_history_all"
+test_dc_history_prereqs="test_dc_history_all"
+
 karatsuba="@printf 'karatsuba cannot be run because one of bc or dc is not built\\\\n'"
 karatsuba_test="@printf 'karatsuba cannot be run because one of bc or dc is not built\\\\n'"
 
@@ -754,12 +940,13 @@ second_target_prereqs=""
 second_target_cmd="$default_target_cmd"
 second_target="\$(BC_EXEC)"
 
+# This if/else if chain is for setting the defaults that change based on whether
+# the library is being built, bc only, dc only, or both calculators.
 if [ "$library" -ne 0 ]; then
 
 	extra_math=1
 	nls=0
 	hist=0
-	prompt=0
 	bc=1
 	dc=1
 
@@ -767,6 +954,8 @@ if [ "$library" -ne 0 ]; then
 	default_target_cmd="ar -r -cu \$(LIBBC) \$(OBJ)"
 	default_target="\$(LIBBC)"
 	tests="test_library"
+	test_bc_history_prereqs=" test_bc_history_skip"
+	test_dc_history_prereqs=" test_dc_history_skip"
 
 elif [ "$bc_only" -eq 1 ]; then
 
@@ -778,6 +967,7 @@ elif [ "$bc_only" -eq 1 ]; then
 	executables="bc"
 
 	dc_test="@printf 'No dc tests to run\\\\n'"
+	test_dc_history_prereqs=" test_dc_history_skip"
 
 	install_prereqs=" install_execs"
 	install_man_prereqs=" install_bc_manpage"
@@ -786,7 +976,7 @@ elif [ "$bc_only" -eq 1 ]; then
 
 	default_target="\$(BC_EXEC)"
 	second_target="\$(DC_EXEC)"
-	tests="test_bc timeconst"
+	tests="test_bc timeconst test_history"
 
 elif [ "$dc_only" -eq 1 ]; then
 
@@ -802,6 +992,7 @@ elif [ "$dc_only" -eq 1 ]; then
 	executable="DC_EXEC"
 
 	bc_test="@printf 'No bc tests to run\\\\n'"
+	test_bc_history_prereqs=" test_bc_history_skip"
 
 	timeconst="@printf 'timeconst cannot be run because bc is not built\\\\n'"
 
@@ -810,7 +1001,7 @@ elif [ "$dc_only" -eq 1 ]; then
 	uninstall_prereqs=" uninstall_dc"
 	uninstall_man_prereqs=" uninstall_dc_manpage"
 
-	tests="test_dc"
+	tests="test_dc test_history"
 
 else
 
@@ -841,14 +1032,15 @@ else
 
 fi
 
+# We need specific stuff for fuzzing.
 if [ "$fuzz" -ne 0 ]; then
 	debug=1
 	hist=0
-	prompt=0
 	nls=0
 	optimization="3"
 fi
 
+# This sets some necessary things for debug mode.
 if [ "$debug" -eq 1 ]; then
 
 	if [ -z "$CFLAGS" ] && [ -z "$optimization" ]; then
@@ -858,16 +1050,20 @@ if [ "$debug" -eq 1 ]; then
 	CFLAGS="-g $CFLAGS"
 
 else
+
 	CPPFLAGS="-DNDEBUG $CPPFLAGS"
+
 	if [ "$strip_bin" -ne 0 ]; then
 		LDFLAGS="-s $LDFLAGS"
 	fi
 fi
 
+# Set optimization CFLAGS.
 if [ -n "$optimization" ]; then
 	CFLAGS="-O$optimization $CFLAGS"
 fi
 
+# Set test coverage defaults.
 if [ "$coverage" -eq 1 ]; then
 
 	if [ "$bc_only" -eq 1 ] || [ "$dc_only" -eq 1 ]; then
@@ -879,7 +1075,7 @@ if [ "$coverage" -eq 1 ]; then
 
 	COVERAGE_OUTPUT="@gcov -pabcdf \$(GCDA) \$(BC_GCDA) \$(DC_GCDA) \$(HISTORY_GCDA) \$(RAND_GCDA)"
 	COVERAGE_OUTPUT="$COVERAGE_OUTPUT;\$(RM) -f \$(GEN)*.gc*"
-	COVERAGE_OUTPUT="$COVERAGE_OUTPUT;gcovr --html-details --output index.html"
+	COVERAGE_OUTPUT="$COVERAGE_OUTPUT;gcovr --exclude-unreachable-branches --exclude-throw-branches --html-details --output index.html"
 	COVERAGE_PREREQS=" test coverage_output"
 
 else
@@ -887,6 +1083,8 @@ else
 	COVERAGE_PREREQS=""
 fi
 
+
+# Set some defaults.
 if [ -z "${DESTDIR+set}" ]; then
 	destdir=""
 else
@@ -909,12 +1107,16 @@ if [ -z "${LIBDIR+set}" ]; then
 	LIBDIR="$PREFIX/lib"
 fi
 
+# Set a default for the DATAROOTDIR. This is done if either manpages will be
+# installed, or locales are enabled because that's probably where NLS_PATH
+# points.
 if [ "$install_manpages" -ne 0 ] || [ "$nls" -ne 0 ]; then
 	if [ -z "${DATAROOTDIR+set}" ]; then
 		DATAROOTDIR="$PREFIX/share"
 	fi
 fi
 
+# Set defaults for manpage environment variables.
 if [ "$install_manpages" -ne 0 ]; then
 
 	if [ -z "${DATADIR+set}" ]; then
@@ -938,6 +1140,9 @@ else
 	uninstall_man_prereqs=""
 fi
 
+# Here is where we test NLS (the locale system). This is done by trying to
+# compile src/vm.c, which has the relevant code. If it fails, then it is
+# disabled.
 if [ "$nls" -ne 0 ]; then
 
 	set +e
@@ -945,7 +1150,7 @@ if [ "$nls" -ne 0 ]; then
 	printf 'Testing NLS...\n'
 
 	flags="-DBC_ENABLE_NLS=1 -DBC_ENABLED=$bc -DDC_ENABLED=$dc"
-	flags="$flags -DBC_ENABLE_HISTORY=$hist"
+	flags="$flags -DBC_ENABLE_HISTORY=$hist -DBC_ENABLE_LIBRARY=0 -DBC_ENABLE_AFL=0"
 	flags="$flags -DBC_ENABLE_EXTRA_MATH=$extra_math -I./include/"
 	flags="$flags -D_POSIX_C_SOURCE=200809L -D_XOPEN_SOURCE=700"
 
@@ -987,7 +1192,10 @@ if [ "$nls" -ne 0 ]; then
 
 			printf 'gencat works.\n\n'
 
-			if [ "$HOSTCC" != "$CC" ]; then
+			# It turns out that POSIX locales are really terrible, and running
+			# gencat on one machine is not guaranteed to make those cat files
+			# portable to another machine, so we had better warn the user here.
+			if [ "$HOSTCC" != "$CC" ] || [ "$OLDHOSTCFLAGS" != "$OLDCFLAGS" ]; then
 				printf 'Cross-compile detected.\n\n'
 				printf 'WARNING: Catalog files generated with gencat may not be portable\n'
 				printf '         across different architectures.\n\n'
@@ -1018,6 +1226,7 @@ else
 	install_locales="\$(LOCALE_INSTALL) \$(NLSPATH) \$(MAIN_EXEC) \$(DESTDIR)"
 fi
 
+# Like the above tested locale support, this tests history.
 if [ "$hist" -eq 1 ]; then
 
 	set +e
@@ -1025,7 +1234,7 @@ if [ "$hist" -eq 1 ]; then
 	printf 'Testing history...\n'
 
 	flags="-DBC_ENABLE_HISTORY=1 -DBC_ENABLED=$bc -DDC_ENABLED=$dc"
-	flags="$flags -DBC_ENABLE_NLS=$nls -DBC_ENABLE_LIBRARY=0"
+	flags="$flags -DBC_ENABLE_NLS=$nls -DBC_ENABLE_LIBRARY=0 -DBC_ENABLE_AFL=0"
 	flags="$flags -DBC_ENABLE_EXTRA_MATH=$extra_math -I./include/"
 	flags="$flags -D_POSIX_C_SOURCE=200809L -D_XOPEN_SOURCE=700"
 
@@ -1053,6 +1262,39 @@ if [ "$hist" -eq 1 ]; then
 
 fi
 
+# We have to disable the history tests if it is disabled or valgrind is on.
+if [ "$hist" -eq 0 ] || [ "$vg" -ne 0 ]; then
+	test_bc_history_prereqs=" test_bc_history_skip"
+	test_dc_history_prereqs=" test_dc_history_skip"
+	history_tests="@printf 'Skipping history tests...\\\\n'"
+else
+	history_tests="@printf '\$(TEST_STARS)\\\\n\\\\nRunning history tests...\\\\n\\\\n' \&\& tests/history.sh bc -a \&\& tests/history.sh dc -a \&\& printf '\\\\nAll history tests passed.\\\\n\\\\n\$(TEST_STARS)\\\\n'"
+fi
+
+# Test OpenBSD. This is not in an if statement because regardless of whatever
+# the user says, we need to know if we are on OpenBSD to activate _BSD_SOURCE.
+# No, I cannot `#define _BSD_SOURCE` in a header because OpenBSD's patched GCC
+# and Clang complain that that is only allowed for system headers. Sigh....So we
+# have to check at configure time and set it on the compiler command-line. And
+# we have to set it because we also set _POSIX_C_SOURCE, which OpenBSD headers
+# detect, and when they detect it, they turn off _BSD_SOURCE unless it is
+# specifically requested.
+set +e
+printf 'Testing for OpenBSD...\n'
+
+flags="-DBC_TEST_OPENBSD -DBC_ENABLE_AFL=0"
+"$CC" $CPPFLAGS $CFLAGS $flags -I./include -E "include/status.h" > /dev/null 2>&1
+
+err="$?"
+
+if [ "$err" -ne 0 ]; then
+	printf 'On OpenBSD. Using _BSD_SOURCE.\n\n'
+	bsd="-D_BSD_SOURCE"
+else
+	printf 'Not on OpenBSD.\n\n'
+	bsd=""
+fi
+
 if [ "$library" -eq 1 ]; then
 	bc_lib=""
 fi
@@ -1063,9 +1305,11 @@ else
 	BC_LIB2_O=""
 fi
 
+# These lines set the appropriate targets based on whether `gen/strgen.c` or
+# `gen/strgen.sh` is used.
 GEN="strgen"
 GEN_EXEC_TARGET="\$(HOSTCC) \$(HOSTCFLAGS) -o \$(GEN_EXEC) \$(GEN_C)"
-CLEAN_PREREQS=" clean_gen"
+CLEAN_PREREQS=" clean_gen clean_coverage"
 
 if [ -z "${GEN_HOST+set}" ]; then
 	GEN_HOST=1
@@ -1073,7 +1317,7 @@ else
 	if [ "$GEN_HOST" -eq 0 ]; then
 		GEN="strgen.sh"
 		GEN_EXEC_TARGET="@printf 'Do not need to build gen/strgen.c\\\\n'"
-		CLEAN_PREREQS=""
+		CLEAN_PREREQS=" clean_coverage"
 	fi
 fi
 
@@ -1081,6 +1325,7 @@ manpage_args=""
 unneeded=""
 headers="\$(HEADERS)"
 
+# This series of if statements figure out what source files are *not* needed.
 if [ "$extra_math" -eq 0 ]; then
 	manpage_args="E"
 	unneeded="$unneeded rand.c"
@@ -1088,6 +1333,9 @@ else
 	headers="$headers \$(EXTRA_MATH_HEADERS)"
 fi
 
+# All of these next if statements set the build type and mark certain source
+# files as unneeded so that they won't have targets generated for them.
+
 if [ "$hist" -eq 0 ]; then
 	manpage_args="${manpage_args}H"
 	unneeded="$unneeded history.c"
@@ -1099,10 +1347,6 @@ if [ "$nls" -eq 0 ]; then
 	manpage_args="${manpage_args}N"
 fi
 
-if [ "$prompt" -eq 0 ]; then
-	manpage_args="${manpage_args}P"
-fi
-
 if [ "$bc" -eq 0 ]; then
 	unneeded="$unneeded bc.c bc_lex.c bc_parse.c"
 else
@@ -1125,6 +1369,12 @@ else
 	unneeded="$unneeded library.c"
 fi
 
+# library.c is not needed under normal circumstances.
+if [ "$unneeded" = "" ]; then
+	unneeded="library.c"
+fi
+
+# This sets the appropriate manpage for a full build.
 if [ "$manpage_args" = "" ]; then
 	manpage_args="A"
 fi
@@ -1133,6 +1383,15 @@ if [ "$vg" -ne 0 ]; then
 	memcheck=1
 fi
 
+if [ "$bc_default_prompt" = "" ]; then
+	bc_default_prompt="$bc_default_tty_mode"
+fi
+
+if [ "$dc_default_prompt" = "" ]; then
+	dc_default_prompt="$dc_default_tty_mode"
+fi
+
+# Generate the test targets and prerequisites.
 bc_tests=$(gen_test_targets bc)
 bc_script_tests=$(gen_script_test_targets bc)
 dc_tests=$(gen_test_targets dc)
@@ -1154,7 +1413,6 @@ printf 'BC_ENABLE_LIBRARY=%s\n\n' "$library"
 printf 'BC_ENABLE_HISTORY=%s\n' "$hist"
 printf 'BC_ENABLE_EXTRA_MATH=%s\n' "$extra_math"
 printf 'BC_ENABLE_NLS=%s\n' "$nls"
-printf 'BC_ENABLE_PROMPT=%s\n' "$prompt"
 printf 'BC_ENABLE_AFL=%s\n' "$fuzz"
 printf '\n'
 printf 'BC_NUM_KARATSUBA_LEN=%s\n' "$karatsuba_len"
@@ -1181,6 +1439,19 @@ printf 'DESTDIR=%s\n' "$DESTDIR"
 printf 'LONG_BIT=%s\n' "$LONG_BIT"
 printf 'GEN_HOST=%s\n' "$GEN_HOST"
 printf 'GEN_EMU=%s\n' "$GEN_EMU"
+printf '\n'
+printf 'Setting Defaults\n'
+printf '================\n'
+printf 'bc.banner=%s\n' "$bc_default_banner"
+printf 'bc.sigint_reset=%s\n' "$bc_default_sigint_reset"
+printf 'dc.sigint_reset=%s\n' "$dc_default_sigint_reset"
+printf 'bc.tty_mode=%s\n' "$bc_default_tty_mode"
+printf 'dc.tty_mode=%s\n' "$dc_default_tty_mode"
+printf 'bc.prompt=%s\n' "$bc_default_prompt"
+printf 'dc.prompt=%s\n' "$dc_default_prompt"
+
+# This is where the real work begins. This is the point at which the Makefile.in
+# template is edited and output to the Makefile.
 
 contents=$(cat "$scriptdir/Makefile.in")
 
@@ -1189,14 +1460,14 @@ replacement='*** WARNING: Autogenerated from Makefile.in. DO NOT MODIFY ***'
 
 contents=$(replace "$contents" "$needle" "$replacement")
 
-if [ "$unneeded" = "" ]; then
-	unneeded="library.c"
-fi
-
+# The contents are edited to have the list of files to build.
 contents=$(gen_file_list "$contents" $unneeded)
 
 SRC_TARGETS=""
 
+# This line and loop generates the individual targets for source files. I used
+# to just use an implicit target, but that was found to be inadequate when I
+# added the library.
 src_files=$(find_src_files $unneeded)
 
 for f in $src_files; do
@@ -1205,6 +1476,7 @@ for f in $src_files; do
 		"$SRC_TARGETS" "$o" "$headers" "$f" "$o" "$f")
 done
 
+# Replace all the placeholders.
 contents=$(replace "$contents" "HEADERS" "$headers")
 
 contents=$(replace "$contents" "BC_ENABLED" "$bc")
@@ -1221,11 +1493,12 @@ contents=$(replace "$contents" "DC_TESTS" "$dc_tests")
 contents=$(replace "$contents" "DC_SCRIPT_TESTS" "$dc_script_tests")
 contents=$(replace "$contents" "DC_TEST_EXEC" "$dc_test_exec")
 
+contents=$(replace "$contents" "BUILD_TYPE" "$manpage_args")
+
 contents=$(replace "$contents" "LIBRARY" "$library")
 contents=$(replace "$contents" "HISTORY" "$hist")
 contents=$(replace "$contents" "EXTRA_MATH" "$extra_math")
 contents=$(replace "$contents" "NLS" "$nls")
-contents=$(replace "$contents" "PROMPT" "$prompt")
 contents=$(replace "$contents" "FUZZ" "$fuzz")
 contents=$(replace "$contents" "MEMCHECK" "$memcheck")
 
@@ -1279,7 +1552,10 @@ contents=$(replace "$contents" "EXEC" "$executable")
 contents=$(replace "$contents" "TESTS" "$tests")
 
 contents=$(replace "$contents" "BC_TEST" "$bc_test")
+contents=$(replace "$contents" "BC_HISTORY_TEST_PREREQS" "$test_bc_history_prereqs")
 contents=$(replace "$contents" "DC_TEST" "$dc_test")
+contents=$(replace "$contents" "DC_HISTORY_TEST_PREREQS" "$test_dc_history_prereqs")
+contents=$(replace "$contents" "HISTORY_TESTS" "$history_tests")
 
 contents=$(replace "$contents" "VG_BC_TEST" "$vg_bc_test")
 contents=$(replace "$contents" "VG_DC_TEST" "$vg_dc_test")
@@ -1297,20 +1573,33 @@ contents=$(replace "$contents" "GEN_EXEC_TARGET" "$GEN_EXEC_TARGET")
 contents=$(replace "$contents" "CLEAN_PREREQS" "$CLEAN_PREREQS")
 contents=$(replace "$contents" "GEN_EMU" "$GEN_EMU")
 
+contents=$(replace "$contents" "BSD" "$bsd")
+
+contents=$(replace "$contents" "BC_DEFAULT_BANNER" "$bc_default_banner")
+contents=$(replace "$contents" "BC_DEFAULT_SIGINT_RESET" "$bc_default_sigint_reset")
+contents=$(replace "$contents" "DC_DEFAULT_SIGINT_RESET" "$dc_default_sigint_reset")
+contents=$(replace "$contents" "BC_DEFAULT_TTY_MODE" "$bc_default_tty_mode")
+contents=$(replace "$contents" "DC_DEFAULT_TTY_MODE" "$dc_default_tty_mode")
+contents=$(replace "$contents" "BC_DEFAULT_PROMPT" "$bc_default_prompt")
+contents=$(replace "$contents" "DC_DEFAULT_PROMPT" "$dc_default_prompt")
+
+# Do the first print to the Makefile.
 printf '%s\n%s\n\n' "$contents" "$SRC_TARGETS" > "$scriptdir/Makefile"
 
+# Generate the individual test targets.
 if [ "$bc" -ne 0 ]; then
-	gen_tests bc BC "$extra_math" "$time_tests" $bc_test_exec
+	gen_tests bc "$extra_math" "$time_tests" $bc_test_exec
 	gen_script_tests bc "$extra_math" "$generate_tests" "$time_tests" $bc_test_exec
 fi
 
 if [ "$dc" -ne 0 ]; then
-	gen_tests dc DC "$extra_math" "$time_tests" $dc_test_exec
+	gen_tests dc "$extra_math" "$time_tests" $dc_test_exec
 	gen_script_tests dc "$extra_math" "$generate_tests" "$time_tests" $dc_test_exec
 fi
 
 cd "$scriptdir"
 
+# Copy the correct manuals to the expected places.
 cp -f manuals/bc/$manpage_args.1.md manuals/bc.1.md
 cp -f manuals/bc/$manpage_args.1 manuals/bc.1
 cp -f manuals/dc/$manpage_args.1.md manuals/dc.1.md
