@@ -36,14 +36,54 @@
 #ifndef BC_BCL_H
 #define BC_BCL_H
 
+#ifdef _WIN32
+#include <Windows.h>
+#include <BaseTsd.h>
+#include <stdio.h>
+#include <io.h>
+#endif // _WIN32
+
 #include <stdbool.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <stdint.h>
 #include <sys/types.h>
 
-#define BC_SEED_ULONGS (4)
-#define BC_SEED_SIZE (sizeof(long) * BC_SEED_ULONGS)
+// Windows has deprecated isatty() and the rest of these. Or doesn't have them.
+// So these are just fixes for Windows.
+#ifdef _WIN32
+
+// This one is special. Windows did not like me defining an
+// inline function that was not given a definition in a header
+// file. This suppresses that by making inline functions non-inline.
+#define inline
+
+#define restrict __restrict
+#define strdup _strdup
+#define write(f, b, s) _write((f), (b), (unsigned int) (s))
+#define read(f, b, s) _read((f), (b), (unsigned int) (s))
+#define close _close
+#define open(f, n, m) _sopen_s(f, n, m, _SH_DENYNO, _S_IREAD | _S_IWRITE)
+#define sigjmp_buf jmp_buf
+#define sigsetjmp(j, s) setjmp(j)
+#define siglongjmp longjmp
+#define isatty _isatty
+#define STDIN_FILENO _fileno(stdin)
+#define STDOUT_FILENO _fileno(stdout)
+#define STDERR_FILENO _fileno(stderr)
+#define ssize_t SSIZE_T
+#define S_ISDIR(m) ((m) & _S_IFDIR)
+#define O_RDONLY _O_RDONLY
+#define stat _stat
+#define fstat _fstat
+#define BC_FILE_SEP '\\'
+
+#else // _WIN32
+#define BC_FILE_SEP '/'
+#endif // _WIN32
+
+#define BCL_SEED_ULONGS (4)
+#define BCL_SEED_SIZE (sizeof(long) * BCL_SEED_ULONGS)
 
 // For some reason, LONG_BIT is not defined in some versions of gcc.
 // I define it here to the minimum accepted value in the POSIX standard.
@@ -59,6 +99,13 @@
 #error BC_LONG_BIT cannot be greater than LONG_BIT
 #endif // BC_LONG_BIT > LONG_BIT
 
+// For more information about the items here, see the either the
+// manuals/bcl.3.md or manuals/bcl.3 manuals.
+
+// BclBigDig is a fixed-size integer type that bcl can convert numbers to.
+//
+// BclRandInt is the type of fixed-size integer natively returned by the
+// pseudo-random number generator.
 #if BC_LONG_BIT >= 64
 
 typedef uint64_t BclBigDig;
@@ -75,6 +122,12 @@ typedef uint32_t BclRandInt;
 
 #endif // BC_LONG_BIT >= 64
 
+#ifndef BC_ENABLE_LIBRARY
+#define BC_ENABLE_LIBRARY (1)
+#endif // BC_ENABLE_LIBRARY
+
+#if BC_ENABLE_LIBRARY
+
 typedef enum BclError {
 
 	BCL_ERROR_NONE,
@@ -175,10 +228,12 @@ BclNumber bcl_frand(size_t places);
 BclNumber bcl_ifrand(BclNumber a, size_t places);
 
 BclError bcl_rand_seedWithNum(BclNumber n);
-BclError bcl_rand_seed(unsigned char seed[BC_SEED_SIZE]);
+BclError bcl_rand_seed(unsigned char seed[BCL_SEED_SIZE]);
 void bcl_rand_reseed(void);
 BclNumber bcl_rand_seed2num(void);
 BclRandInt bcl_rand_int(void);
 BclRandInt bcl_rand_bounded(BclRandInt bound);
 
+#endif // BC_ENABLE_LIBRARY
+
 #endif // BC_BCL_H
