@@ -40,6 +40,11 @@
 
 #include <num.h>
 
+/**
+ * A header for functions that need to lock and setjmp(). It also sets the
+ * variable that tells bcl that it is running.
+ * @param l  The label to jump to on error.
+ */
 #define BC_FUNC_HEADER_LOCK(l)   \
 	do {                         \
 		BC_SIG_LOCK;             \
@@ -48,6 +53,11 @@
 		vm.running = 1;          \
 	} while (0)
 
+/**
+ * A footer to unlock and stop the jumping if an error happened. It also sets
+ * the variable that tells bcl that it is running.
+ * @param e  The error variable to set.
+ */
 #define BC_FUNC_FOOTER_UNLOCK(e) \
 	do {                         \
 		BC_SIG_ASSERT_LOCKED;    \
@@ -58,6 +68,10 @@
 		vm.sig_lock = 0;         \
 	} while (0)
 
+/**
+ * A header that sets a jump and sets running.
+ * @param l  The label to jump to on error.
+ */
 #define BC_FUNC_HEADER(l)        \
 	do {                         \
 		BC_SETJMP(l);            \
@@ -65,6 +79,11 @@
 		vm.running = 1;          \
 	} while (0)
 
+/**
+ * A header that assumes that signals are already locked. It sets a jump and
+ * running.
+ * @param l  The label to jump to on error.
+ */
 #define BC_FUNC_HEADER_INIT(l)   \
 	do {                         \
 		BC_SETJMP_LOCKED(l);     \
@@ -72,6 +91,10 @@
 		vm.running = 1;          \
 	} while (0)
 
+/**
+ * A footer for functions that do not return an error code. It clears running
+ * and unlocks the signals. It also stops the jumping.
+ */
 #define BC_FUNC_FOOTER_NO_ERR \
 	do {                      \
 		vm.running = 0;       \
@@ -80,19 +103,25 @@
 		vm.sig_lock = 0;      \
 	} while (0)
 
+/**
+ * A footer for functions that *do* return an error code. It clears running and
+ * unlocks the signals. It also stops the jumping.
+ * @param e  The error variable to set.
+ */
 #define BC_FUNC_FOOTER(e)      \
 	do {                       \
 		e = vm.err;            \
 		BC_FUNC_FOOTER_NO_ERR; \
 	} while (0)
 
-#define BC_FUNC_RESETJMP(l)   \
-	do {                      \
-		BC_SIG_ASSERT_LOCKED; \
-		BC_UNSETJMP;          \
-		BC_SETJMP_LOCKED(l);  \
-	} while (0)
-
+/**
+ * A footer that sets up n based the value of e and sets up the return value in
+ * idx.
+ * @param c    The context.
+ * @param e    The error.
+ * @param n    The number.
+ * @param idx  The idx to set as the return value.
+ */
 #define BC_MAYBE_SETUP(c, e, n, idx)                \
 	do {                                            \
 		if (BC_ERR((e) != BCL_ERROR_NONE)) {        \
@@ -102,6 +131,11 @@
 		else idx = bcl_num_insert(c, &(n));         \
 	} while (0)
 
+/**
+ * A header to check the context and return an error encoded in a number if it
+ * is bad.
+ * @param c  The context.
+ */
 #define BC_CHECK_CTXT(c)                                      \
 	do {                                                      \
 		c = bcl_context();                                    \
@@ -112,6 +146,11 @@
 		}                                                     \
 	} while (0)
 
+
+/**
+ * A header to check the context and return an error directly if it is bad.
+ * @param c  The context.
+ */
 #define BC_CHECK_CTXT_ERR(c)                  \
 	do {                                      \
 		c = bcl_context();                    \
@@ -120,12 +159,22 @@
 		}                                     \
 	} while (0)
 
+/**
+ * A header to check the context and abort if it is bad.
+ * @param c  The context.
+ */
 #define BC_CHECK_CTXT_ASSERT(c) \
 	do {                        \
 		c = bcl_context();      \
 		assert(c != NULL);      \
 	} while (0)
 
+/**
+ * A header to check the number in the context and return an error encoded as a
+ * @param c  The context.
+ * number if it is bad.
+ * @param n  The BclNumber.
+ */
 #define BC_CHECK_NUM(c, n)                                         \
 	do {                                                           \
 		if (BC_ERR((n).i >= (c)->nums.len)) {                      \
@@ -138,6 +187,12 @@
 		}                                                          \
 	} while (0)
 
+/**
+ * A header to check the number in the context and return an error directly if
+ * it is bad.
+ * @param c  The context.
+ * @param n  The BclNumber.
+ */
 #define BC_CHECK_NUM_ERR(c, n)                         \
 	do {                                               \
 		if (BC_ERR((n).i >= (c)->nums.len)) {          \
@@ -147,17 +202,36 @@
 		}                                              \
 	} while (0)
 
+/**
+ * Turns a BclNumber into a BcNum.
+ * @param c  The context.
+ * @param n  The BclNumber.
+ */
 #define BC_NUM(c, n) ((BcNum*) bc_vec_item(&(c)->nums, (n).i))
 
-typedef size_t (*BcReqOp)(const BcNum*, const BcNum*, size_t);
+/**
+ * Frees a BcNum for bcl. This is a destructor.
+ * @param num  The BcNum to free, as a void pointer.
+ */
+void bcl_num_destruct(void *num);
 
+/// The actual context struct.
 typedef struct BclCtxt {
 
+	/// The context's scale.
 	size_t scale;
+
+	/// The context's ibase.
 	size_t ibase;
+
+	/// The context's obase.
 	size_t obase;
 
+	/// A vector of BcNum numbers.
 	BcVec nums;
+
+	/// A vector of BclNumbers. These are the indices in nums that are currently
+	/// not used (because they were freed).
 	BcVec free_nums;
 
 } BclCtxt;
