@@ -33,6 +33,9 @@
  *
  */
 
+#include <assert.h>
+
+#include <opt.h>
 #include <args.h>
 #include <lex.h>
 #include <parse.h>
@@ -41,27 +44,129 @@
 #include <num.h>
 #include <rand.h>
 #include <program.h>
+#include <history.h>
+#include <library.h>
 #include <vm.h>
 
 #if !BC_ENABLE_LIBRARY
 
 #if BC_ENABLED
+
+/// The bc signal message and its length.
 const char bc_sig_msg[] = "\ninterrupt (type \"quit\" to exit)\n";
 const uchar bc_sig_msg_len = (uchar) (sizeof(bc_sig_msg) - 1);
+
 #endif // BC_ENABLED
+
 #if DC_ENABLED
+
+/// The dc signal message and its length.
 const char dc_sig_msg[] = "\ninterrupt (type \"q\" to exit)\n";
 const uchar dc_sig_msg_len = (uchar) (sizeof(dc_sig_msg) - 1);
+
 #endif // DC_ENABLED
 
+/// The copyright banner.
 const char bc_copyright[] =
 	"Copyright (c) 2018-2021 Gavin D. Howard and contributors\n"
 	"Report bugs at: https://git.yzena.com/gavin/bc\n\n"
 	"This is free software with ABSOLUTELY NO WARRANTY.\n";
 
+#ifdef __OpenBSD__
+
+#if BC_ENABLE_EXTRA_MATH
+
+#if BC_ENABLE_HISTORY
+
+/// The pledges for starting bc.
+const char bc_pledge_start[] = "rpath stdio tty unveil";
+
+/// The final pledges with history enabled.
+const char bc_pledge_end_history[] = "rpath stdio tty";
+
+#else // BC_ENABLE_HISTORY
+
+/// The pledges for starting bc.
+const char bc_pledge_start[] = "rpath stdio unveil";
+
+#endif // BC_ENABLE_HISTORY
+
+/// The final pledges with history history disabled.
+const char bc_pledge_end[] = "rpath stdio";
+
+#else // BC_ENABLE_EXTRA_MATH
+
+#if BC_ENABLE_HISTORY
+
+/// The pledges for starting bc.
+const char bc_pledge_start[] = "rpath stdio tty";
+
+/// The final pledges with history enabled.
+const char bc_pledge_end_history[] = "stdio tty";
+
+#else // BC_ENABLE_HISTORY
+
+/// The pledges for starting bc.
+const char bc_pledge_start[] = "rpath stdio";
+
+#endif // BC_ENABLE_HISTORY
+
+/// The final pledges with history history disabled.
+const char bc_pledge_end[] = "stdio";
+
+#endif // BC_ENABLE_EXTRA_MATH
+
+#else // __OpenBSD__
+
+/// The pledges for starting bc.
+const char bc_pledge_start[] = "";
+
+#if BC_ENABLE_HISTORY
+
+/// The final pledges with history enabled.
+const char bc_pledge_end_history[] = "";
+
+#endif // BC_ENABLE_HISTORY
+
+/// The final pledges with history history disabled.
+const char bc_pledge_end[] = "";
+
+#endif // __OpenBSD__
+
+/// The list of long options. There is a zero set at the end for detecting the
+/// end.
+const BcOptLong bc_args_lopt[] = {
+
+	{ "expression", BC_OPT_REQUIRED, 'e' },
+	{ "file", BC_OPT_REQUIRED, 'f' },
+	{ "help", BC_OPT_NONE, 'h' },
+	{ "interactive", BC_OPT_NONE, 'i' },
+	{ "no-prompt", BC_OPT_NONE, 'P' },
+	{ "no-read-prompt", BC_OPT_NONE, 'R' },
+#if BC_ENABLED
+	{ "global-stacks", BC_OPT_BC_ONLY, 'g' },
+	{ "mathlib", BC_OPT_BC_ONLY, 'l' },
+	{ "quiet", BC_OPT_BC_ONLY, 'q' },
+	{ "redefine", BC_OPT_REQUIRED_BC_ONLY, 'r' },
+	{ "standard", BC_OPT_BC_ONLY, 's' },
+	{ "warn", BC_OPT_BC_ONLY, 'w' },
+#endif // BC_ENABLED
+	{ "version", BC_OPT_NONE, 'v' },
+	{ "version", BC_OPT_NONE, 'V' },
+#if DC_ENABLED
+	{ "extended-register", BC_OPT_DC_ONLY, 'x' },
+#endif // DC_ENABLED
+	{ NULL, 0, 0 },
+
+};
+
+/// The function header for error messages.
 const char* const bc_err_func_header = "Function:";
+
+/// The line format string for error messages.
 const char* const bc_err_line = ":%zu";
 
+/// The default error category strings.
 const char *bc_errs[] = {
 	"Math error:",
 	"Parse error:",
@@ -72,16 +177,18 @@ const char *bc_errs[] = {
 #endif // BC_ENABLED
 };
 
+/// The error category for each error.
 const uchar bc_err_ids[] = {
 
 	BC_ERR_IDX_MATH, BC_ERR_IDX_MATH, BC_ERR_IDX_MATH, BC_ERR_IDX_MATH,
 
 	BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL,
 	BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL, BC_ERR_IDX_FATAL,
+	BC_ERR_IDX_FATAL,
 
 	BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC,
 	BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC,
-	BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC,
+	BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC, BC_ERR_IDX_EXEC,
 
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 	BC_ERR_IDX_PARSE,
@@ -93,11 +200,13 @@ const uchar bc_err_ids[] = {
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
-	BC_ERR_IDX_PARSE,
+	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 #endif // BC_ENABLED
 
 };
 
+/// The default error messages. There are NULL pointers because the positions
+/// must be preserved for the locales.
 const char* const bc_err_msgs[] = {
 
 	"negative number",
@@ -108,11 +217,12 @@ const char* const bc_err_msgs[] = {
 	"memory allocation failed",
 	"I/O error",
 	"cannot open file: %s",
-	"file is not ASCII: %s",
+	"file is not text: %s",
 	"path is a directory: %s",
 	"bad command-line option: \"%s\"",
 	"option requires an argument: '%c' (\"%s\")",
 	"option takes no arguments: '%c' (\"%s\")",
+	"bad option argument: \"%s\"",
 
 	"bad ibase: must be [%lu, %lu]",
 	"bad obase: must be [%lu, %lu]",
@@ -122,8 +232,9 @@ const char* const bc_err_msgs[] = {
 	"variable or array element is the wrong type",
 #if DC_ENABLED
 	"stack has too few elements",
+	"stack for register \"%s\" has too few elements",
 #else // DC_ENABLED
-	NULL,
+	NULL, NULL,
 #endif // DC_ENABLED
 #if BC_ENABLED
 	"wrong number of parameters; need %zu, have %zu",
@@ -141,7 +252,7 @@ const char* const bc_err_msgs[] = {
 #if BC_ENABLED
 	"bad expression",
 	"empty expression",
-	"bad print statement",
+	"bad print or stream statement",
 	"bad function definition",
 	("bad assignment: left side must be scale, ibase, "
 		"obase, seed, last, var, or array element"),
@@ -168,17 +279,58 @@ const char* const bc_err_msgs[] = {
 	"POSIX does not allow array references as function parameters",
 	"POSIX does not allow void functions",
 	"POSIX requires the left brace be on the same line as the function header",
+	"POSIX does not allow strings to be assigned to variables or arrays",
 #endif // BC_ENABLED
 
 };
 
+#endif // !BC_ENABLE_LIBRARY
+
+/// The destructors corresponding to BcDtorType enum items.
+const BcVecFree bc_vec_dtors[] = {
+	NULL,
+	bc_vec_free,
+	bc_num_free,
+#if !BC_ENABLE_LIBRARY
+#ifndef NDEBUG
+	bc_func_free,
+#endif // NDEBUG
+	bc_slab_free,
+	bc_const_free,
+	bc_result_free,
 #if BC_ENABLE_HISTORY
+	bc_history_string_free,
+#endif // BC_ENABLE_HISTORY
+#else // !BC_ENABLE_LIBRARY
+	bcl_num_destruct,
+#endif // !BC_ENABLE_LIBRARY
+};
+
+#if !BC_ENABLE_LIBRARY
+
+#if BC_ENABLE_HISTORY
+
+/// A flush type for not clearing current extras but not saving new ones either.
+const BcFlushType bc_flush_none = BC_FLUSH_NO_EXTRAS_NO_CLEAR;
+
+/// A flush type for clearing extras and not saving new ones.
+const BcFlushType bc_flush_err = BC_FLUSH_NO_EXTRAS_CLEAR;
+
+/// A flush type for clearing previous extras and saving new ones.
+const BcFlushType bc_flush_save = BC_FLUSH_SAVE_EXTRAS_CLEAR;
+#endif // BC_ENABLE_HISTORY
+
+#if BC_ENABLE_HISTORY
+
+/// A list of known bad terminals.
 const char *bc_history_bad_terms[] = { "dumb", "cons25", "emacs", NULL };
 
+/// A constant for tabs and its length. My tab handling is dumb and always
+/// outputs the entire thing.
 const char bc_history_tab[] = "        ";
 const size_t bc_history_tab_len = sizeof(bc_history_tab) - 1;
 
-// These are listed in ascending order for efficiency.
+/// A list of wide chars. These are listed in ascending order for efficiency.
 const uint32_t bc_history_wchars[][2] = {
 	{ 0x1100, 0x115F },
 	{ 0x231A, 0x231B },
@@ -286,10 +438,12 @@ const uint32_t bc_history_wchars[][2] = {
 	{ 0x30000, 0x3FFFD },
 };
 
+/// The length of the wide chars list.
 const size_t bc_history_wchars_len =
 	sizeof(bc_history_wchars) / sizeof(bc_history_wchars[0]);
 
-// These are listed in ascending order for efficiency.
+/// A list of combining characters in Unicode. These are listed in ascending
+/// order for efficiency.
 const uint32_t bc_history_combo_chars[] = {
 	0x0300,0x0301,0x0302,0x0303,0x0304,0x0305,0x0306,0x0307,
 	0x0308,0x0309,0x030A,0x030B,0x030C,0x030D,0x030E,0x030F,
@@ -505,19 +659,20 @@ const uint32_t bc_history_combo_chars[] = {
 	0xE01EC,0xE01ED,0xE01EE,0xE01EF,
 };
 
+/// The length of the combining characters list.
 const size_t bc_history_combo_chars_len =
 	sizeof(bc_history_combo_chars) / sizeof(bc_history_combo_chars[0]);
-
-#if BC_DEBUG_CODE
-BcFile bc_history_debug_fp;
-char *bc_history_debug_buf;
-#endif // BC_DEBUG_CODE
 #endif // BC_ENABLE_HISTORY
 
+/// The human-readable name of the main function in bc source code.
 const char bc_func_main[] = "(main)";
+
+/// The human-readable name of the read function in bc source code.
 const char bc_func_read[] = "(read)";
 
 #if BC_DEBUG_CODE
+
+/// A list of names of instructions for easy debugging output.
 const char* bc_inst_names[] = {
 
 #if BC_ENABLED
@@ -586,9 +741,7 @@ const char* bc_inst_names[] = {
 	"BC_INST_NUM",
 	"BC_INST_VAR",
 	"BC_INST_ARRAY_ELEM",
-#if BC_ENABLED
 	"BC_INST_ARRAY",
-#endif // BC_ENABLED
 
 	"BC_INST_ZERO",
 	"BC_INST_ONE",
@@ -609,6 +762,7 @@ const char* bc_inst_names[] = {
 #if BC_ENABLE_EXTRA_MATH
 	"BC_INST_IRAND",
 #endif // BC_ENABLE_EXTRA_MATH
+	"BC_INST_ASCIIFY",
 	"BC_INST_READ",
 #if BC_ENABLE_EXTRA_MATH
 	"BC_INST_RAND",
@@ -623,9 +777,9 @@ const char* bc_inst_names[] = {
 	"BC_INST_PRINT",
 	"BC_INST_PRINT_POP",
 	"BC_INST_STR",
+#if BC_ENABLED
 	"BC_INST_PRINT_STR",
 
-#if BC_ENABLED
 	"BC_INST_JUMP",
 	"BC_INST_JUMP_ZERO",
 
@@ -638,23 +792,23 @@ const char* bc_inst_names[] = {
 	"BC_INST_HALT",
 #endif // BC_ENABLED
 
-#if DC_ENABLED
 	"BC_INST_POP",
-	"BC_INST_POP_EXEC",
+	"BC_INST_SWAP",
 	"BC_INST_MODEXP",
 	"BC_INST_DIVMOD",
+	"BC_INST_PRINT_STREAM",
+
+#if DC_ENABLED
+	"BC_INST_POP_EXEC",
 
 	"BC_INST_EXECUTE",
 	"BC_INST_EXEC_COND",
 
-	"BC_INST_ASCIIFY",
-	"BC_INST_PRINT_STREAM",
-
 	"BC_INST_PRINT_STACK",
 	"BC_INST_CLEAR_STACK",
+	"BC_INST_REG_STACK_LEN",
 	"BC_INST_STACK_LEN",
 	"BC_INST_DUPLICATE",
-	"BC_INST_SWAP",
 
 	"BC_INST_LOAD",
 	"BC_INST_PUSH_VAR",
@@ -662,14 +816,24 @@ const char* bc_inst_names[] = {
 
 	"BC_INST_QUIT",
 	"BC_INST_NQUIT",
+
+	"BC_INST_EXEC_STACK_LEN",
 #endif // DC_ENABLED
+
+	"BC_INST_INVALID",
 };
+
 #endif // BC_DEBUG_CODE
 
+/// A constant string for 0.
 const char bc_parse_zero[2] = "0";
+
+/// A constant string for 1.
 const char bc_parse_one[2] = "1";
 
 #if BC_ENABLED
+
+/// A list of keywords for bc. This needs to be updated if keywords change.
 const BcLexKeyword bc_lex_kws[] = {
 	BC_LEX_KW_ENTRY("auto", 4, true),
 	BC_LEX_KW_ENTRY("break", 5, true),
@@ -685,63 +849,111 @@ const BcLexKeyword bc_lex_kws[] = {
 	BC_LEX_KW_ENTRY("ibase", 5, true),
 	BC_LEX_KW_ENTRY("obase", 5, true),
 	BC_LEX_KW_ENTRY("scale", 5, true),
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("seed", 4, false),
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("length", 6, true),
 	BC_LEX_KW_ENTRY("print", 5, false),
 	BC_LEX_KW_ENTRY("sqrt", 4, true),
 	BC_LEX_KW_ENTRY("abs", 3, false),
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("irand", 5, false),
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
+	BC_LEX_KW_ENTRY("asciify", 7, false),
+	BC_LEX_KW_ENTRY("modexp", 6, false),
+	BC_LEX_KW_ENTRY("divmod", 6, false),
 	BC_LEX_KW_ENTRY("quit", 4, true),
 	BC_LEX_KW_ENTRY("read", 4, false),
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("rand", 4, false),
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("maxibase", 8, false),
 	BC_LEX_KW_ENTRY("maxobase", 8, false),
 	BC_LEX_KW_ENTRY("maxscale", 8, false),
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_ENTRY("maxrand", 7, false),
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
+	BC_LEX_KW_ENTRY("stream", 6, false),
 	BC_LEX_KW_ENTRY("else", 4, false),
 };
 
+/// The length of the list of bc keywords.
 const size_t bc_lex_kws_len = sizeof(bc_lex_kws) / sizeof(BcLexKeyword);
 
-// This is an array that corresponds to token types. An entry is
-// true if the token is valid in an expression, false otherwise.
+#if BC_C11
+
+// This is here to ensure that BC_LEX_NKWS, which is needed for the
+// redefined_kws in BcVm, is correct. If it's correct under C11, it will be
+// correct under C99, and I did not know any other way of ensuring they remained
+// synchronized.
+static_assert(sizeof(bc_lex_kws) / sizeof(BcLexKeyword) == BC_LEX_NKWS,
+              "BC_LEX_NKWS is wrong.");
+
+#endif // BC_C11
+
+/// An array of booleans that correspond to token types. An entry is true if the
+/// token is valid in an expression, false otherwise. This will need to change
+/// if tokens change.
 const uint8_t bc_parse_exprs[] = {
+
+	// Starts with BC_LEX_EOF.
 	BC_PARSE_EXPR_ENTRY(false, false, true, true, true, true, true, true),
+
+	// Starts with BC_LEX_OP_MULTIPLY if extra math is enabled, BC_LEX_OP_DIVIDE
+	// otherwise.
 	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, true, true, true),
+
+	// Starts with BC_LEX_OP_REL_EQ if extra math is enabled, BC_LEX_OP_REL_LT
+	// otherwise.
 	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, true, true, true),
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+
+#if BC_ENABLE_EXTRA_MATH
+
+	// Starts with BC_LEX_OP_ASSIGN_POWER.
 	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, true, true, true),
+
+	// Starts with BC_LEX_OP_ASSIGN_RSHIFT.
 	BC_PARSE_EXPR_ENTRY(true, true, false, false, true, true, false, false),
-	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, true, true, false),
+
+	// Starts with BC_LEX_RBRACKET.
+	BC_PARSE_EXPR_ENTRY(false, false, false, false, true, true, true, false),
+
+	// Starts with BC_LEX_KW_BREAK.
 	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, false, false, false),
+
+	// Starts with BC_LEX_KW_HALT.
 	BC_PARSE_EXPR_ENTRY(false, true, true, true, true, true, true, false),
-	BC_PARSE_EXPR_ENTRY(true, true, true, false, true, true, true, true),
-	BC_PARSE_EXPR_ENTRY(true, true, false, 0, 0, 0, 0, 0)
-#elif BC_ENABLE_EXTRA_MATH // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
-	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, true, true, true),
-	BC_PARSE_EXPR_ENTRY(true, true, false, false, true, true, false, false),
-	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, true, true, false),
-	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, false, false, false),
-	BC_PARSE_EXPR_ENTRY(false, true, true, true, true, true, false, true),
-	BC_PARSE_EXPR_ENTRY(true, false, true, true, true, true, false, 0),
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+
+	// Starts with BC_LEX_KW_SQRT.
+	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, true, false, true),
+
+	// Starts with BC_LEX_KW_MAXIBASE.
+	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, false, false, 0)
+
+#else // BC_ENABLE_EXTRA_MATH
+
+	// Starts with BC_LEX_OP_ASSIGN_PLUS.
 	BC_PARSE_EXPR_ENTRY(true, true, true, false, false, true, true, false),
-	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, false, true, true),
+
+	// Starts with BC_LEX_COMMA.
+	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, true, true, true),
+
+	// Starts with BC_LEX_KW_AUTO.
 	BC_PARSE_EXPR_ENTRY(false, false, false, false, false, false, false, false),
+
+	// Starts with BC_LEX_KW_WHILE.
 	BC_PARSE_EXPR_ENTRY(false, false, true, true, true, true, true, false),
-	BC_PARSE_EXPR_ENTRY(true, true, false, true, true, true, true, false)
+
+	// Starts with BC_LEX_KW_SQRT.
+	BC_PARSE_EXPR_ENTRY(true, true, true, true, true, false, true, true),
+
+	// Starts with BC_LEX_KW_MAXSCALE,
+	BC_PARSE_EXPR_ENTRY(true, true, false, false, 0, 0, 0, 0)
+
 #endif // BC_ENABLE_EXTRA_MATH
 };
 
-// This is an array of data for operators that correspond to token types.
+/// An array of data for operators that correspond to token types.
 const uchar bc_parse_ops[] = {
 	BC_PARSE_OP(0, false), BC_PARSE_OP(0, false),
 	BC_PARSE_OP(1, false), BC_PARSE_OP(1, false),
@@ -767,35 +979,68 @@ const uchar bc_parse_ops[] = {
 };
 
 // These identify what tokens can come after expressions in certain cases.
+
+/// The valid next tokens for normal expressions.
 const BcParseNext bc_parse_next_expr =
 	BC_PARSE_NEXT(4, BC_LEX_NLINE, BC_LEX_SCOLON, BC_LEX_RBRACE, BC_LEX_EOF);
-const BcParseNext bc_parse_next_param =
+
+/// The valid next tokens for function argument expressions.
+const BcParseNext bc_parse_next_arg =
 	BC_PARSE_NEXT(2, BC_LEX_RPAREN, BC_LEX_COMMA);
+
+/// The valid next tokens for expressions in print statements.
 const BcParseNext bc_parse_next_print =
 	BC_PARSE_NEXT(4, BC_LEX_COMMA, BC_LEX_NLINE, BC_LEX_SCOLON, BC_LEX_EOF);
+
+/// The valid next tokens for if statement conditions or loop conditions. This
+/// is used in for loops for the update expression and for builtin function.
+///
+/// The name is an artifact of history, and is related to @a BC_PARSE_REL (see
+/// include/parse.h). It refers to how POSIX only allows some operators as part
+/// of the conditional of for loops, while loops, and if statements.
 const BcParseNext bc_parse_next_rel = BC_PARSE_NEXT(1, BC_LEX_RPAREN);
+
+/// The valid next tokens for array element expressions.
 const BcParseNext bc_parse_next_elem = BC_PARSE_NEXT(1, BC_LEX_RBRACKET);
+
+/// The valid next tokens for for loop initialization expressions and condition
+/// expressions.
 const BcParseNext bc_parse_next_for = BC_PARSE_NEXT(1, BC_LEX_SCOLON);
+
+/// The valid next tokens for read expressions.
 const BcParseNext bc_parse_next_read =
 	BC_PARSE_NEXT(2, BC_LEX_NLINE, BC_LEX_EOF);
+
+/// The valid next tokens for the arguments of a builtin function with multiple
+/// arguments.
+const BcParseNext bc_parse_next_builtin = BC_PARSE_NEXT(1, BC_LEX_COMMA);
+
 #endif // BC_ENABLED
 
 #if DC_ENABLED
+
+/// A list of instructions that need register arguments in dc.
 const uint8_t dc_lex_regs[] = {
 	BC_LEX_OP_REL_EQ, BC_LEX_OP_REL_LE, BC_LEX_OP_REL_GE, BC_LEX_OP_REL_NE,
 	BC_LEX_OP_REL_LT, BC_LEX_OP_REL_GT, BC_LEX_SCOLON, BC_LEX_COLON,
 	BC_LEX_KW_ELSE, BC_LEX_LOAD, BC_LEX_LOAD_POP, BC_LEX_OP_ASSIGN,
-	BC_LEX_STORE_PUSH,
+	BC_LEX_STORE_PUSH, BC_LEX_REG_STACK_LEVEL, BC_LEX_ARRAY_LENGTH,
 };
 
+/// The length of the list of register instructions.
 const size_t dc_lex_regs_len = sizeof(dc_lex_regs) / sizeof(uint8_t);
 
+/// A list corresponding to characters starting at double quote ("). If an entry
+/// is BC_LEX_INVALID, then that character needs extra lexing in dc. If it does
+/// not, the character can trivially be replaced by the entry. Positions are
+/// kept because it corresponds to the ASCII table. This may need to be changed
+/// if tokens change.
 const uchar dc_lex_tokens[] = {
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_IRAND,
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#else // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
 #if BC_ENABLE_EXTRA_MATH
 	BC_LEX_OP_TRUNC,
@@ -803,13 +1048,13 @@ const uchar dc_lex_tokens[] = {
 	BC_LEX_INVALID,
 #endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_OP_MODULUS, BC_LEX_INVALID,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_RAND,
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#else // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_LPAREN, BC_LEX_RPAREN, BC_LEX_OP_MULTIPLY, BC_LEX_OP_PLUS,
-	BC_LEX_INVALID, BC_LEX_OP_MINUS, BC_LEX_INVALID, BC_LEX_OP_DIVIDE,
+	BC_LEX_EXEC_STACK_LENGTH, BC_LEX_OP_MINUS, BC_LEX_INVALID, BC_LEX_OP_DIVIDE,
 	BC_LEX_INVALID, BC_LEX_INVALID, BC_LEX_INVALID, BC_LEX_INVALID,
 	BC_LEX_INVALID, BC_LEX_INVALID, BC_LEX_INVALID, BC_LEX_INVALID,
 	BC_LEX_INVALID, BC_LEX_INVALID,
@@ -828,24 +1073,24 @@ const uchar dc_lex_tokens[] = {
 	BC_LEX_INVALID,
 #endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_IBASE,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_SEED,
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#else // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_SCALE, BC_LEX_LOAD_POP, BC_LEX_OP_BOOL_AND, BC_LEX_OP_BOOL_NOT,
-	BC_LEX_KW_OBASE, BC_LEX_PRINT_STREAM, BC_LEX_NQUIT, BC_LEX_POP,
+	BC_LEX_KW_OBASE, BC_LEX_KW_STREAM, BC_LEX_NQUIT, BC_LEX_POP,
 	BC_LEX_STORE_PUSH, BC_LEX_KW_MAXIBASE, BC_LEX_KW_MAXOBASE,
 	BC_LEX_KW_MAXSCALE,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_KW_MAXRAND,
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#else // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
-	BC_LEX_SCALE_FACTOR,
-	BC_LEX_INVALID, BC_LEX_KW_LENGTH, BC_LEX_INVALID, BC_LEX_INVALID,
-	BC_LEX_INVALID, BC_LEX_OP_POWER, BC_LEX_NEG, BC_LEX_INVALID,
-	BC_LEX_ASCIIFY, BC_LEX_KW_ABS, BC_LEX_CLEAR_STACK, BC_LEX_DUPLICATE,
+#endif // BC_ENABLE_EXTRA_MATH
+	BC_LEX_SCALE_FACTOR, BC_LEX_ARRAY_LENGTH, BC_LEX_KW_LENGTH,
+	BC_LEX_INVALID, BC_LEX_INVALID, BC_LEX_INVALID,
+	BC_LEX_OP_POWER, BC_LEX_NEG, BC_LEX_INVALID,
+	BC_LEX_KW_ASCIIFY, BC_LEX_KW_ABS, BC_LEX_CLEAR_STACK, BC_LEX_DUPLICATE,
 	BC_LEX_KW_ELSE, BC_LEX_PRINT_STACK, BC_LEX_INVALID,
 #if BC_ENABLE_EXTRA_MATH
 	BC_LEX_OP_RSHIFT,
@@ -853,20 +1098,24 @@ const uchar dc_lex_tokens[] = {
 	BC_LEX_INVALID,
 #endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_STORE_IBASE,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_LEX_STORE_SEED,
-#else // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#else // BC_ENABLE_EXTRA_MATH
 	BC_LEX_INVALID,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_LEX_STORE_SCALE, BC_LEX_LOAD,
 	BC_LEX_OP_BOOL_OR, BC_LEX_PRINT_POP, BC_LEX_STORE_OBASE, BC_LEX_KW_PRINT,
 	BC_LEX_KW_QUIT, BC_LEX_SWAP, BC_LEX_OP_ASSIGN, BC_LEX_INVALID,
 	BC_LEX_INVALID, BC_LEX_KW_SQRT, BC_LEX_INVALID, BC_LEX_EXECUTE,
-	BC_LEX_INVALID, BC_LEX_STACK_LEVEL,
-	BC_LEX_LBRACE, BC_LEX_OP_MODEXP, BC_LEX_RBRACE, BC_LEX_OP_DIVMOD,
+	BC_LEX_REG_STACK_LEVEL, BC_LEX_STACK_LEVEL,
+	BC_LEX_LBRACE, BC_LEX_KW_MODEXP, BC_LEX_RBRACE, BC_LEX_KW_DIVMOD,
 	BC_LEX_INVALID
 };
 
+/// A list of instructions that correspond to lex tokens. If an entry is
+/// BC_INST_INVALID, that lex token needs extra parsing in the dc parser.
+/// Otherwise, the token can trivially be replaced by the entry. This needs to
+/// be updated if the tokens change.
 const uchar dc_parse_insts[] = {
 	BC_INST_INVALID, BC_INST_INVALID,
 #if BC_ENABLED
@@ -903,51 +1152,58 @@ const uchar dc_parse_insts[] = {
 	BC_INST_INVALID, BC_INST_INVALID, BC_INST_INVALID,
 #endif // BC_ENABLED
 	BC_INST_IBASE, BC_INST_OBASE, BC_INST_SCALE,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_INST_SEED,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_INST_LENGTH, BC_INST_PRINT,
 	BC_INST_SQRT, BC_INST_ABS,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_INST_IRAND,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
+	BC_INST_ASCIIFY, BC_INST_MODEXP, BC_INST_DIVMOD,
 	BC_INST_QUIT, BC_INST_INVALID,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_INST_RAND,
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 	BC_INST_MAXIBASE,
 	BC_INST_MAXOBASE, BC_INST_MAXSCALE,
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 	BC_INST_MAXRAND,
 #endif // BC_ENABLE_EXTRA_MATH
-	BC_INST_INVALID,
-	BC_INST_REL_EQ, BC_INST_MODEXP, BC_INST_DIVMOD, BC_INST_INVALID,
+	BC_INST_PRINT_STREAM, BC_INST_INVALID,
+	BC_INST_REL_EQ, BC_INST_INVALID,
 	BC_INST_EXECUTE, BC_INST_PRINT_STACK, BC_INST_CLEAR_STACK,
-	BC_INST_STACK_LEN, BC_INST_DUPLICATE, BC_INST_SWAP, BC_INST_POP,
-	BC_INST_ASCIIFY, BC_INST_PRINT_STREAM,
+	BC_INST_INVALID, BC_INST_STACK_LEN, BC_INST_DUPLICATE, BC_INST_SWAP,
+	BC_INST_POP,
 	BC_INST_INVALID, BC_INST_INVALID, BC_INST_INVALID,
 #if BC_ENABLE_EXTRA_MATH
 	BC_INST_INVALID,
 #endif // BC_ENABLE_EXTRA_MATH
 	BC_INST_INVALID, BC_INST_INVALID, BC_INST_INVALID,
-	BC_INST_PRINT_POP, BC_INST_NQUIT, BC_INST_SCALE_FUNC,
+	BC_INST_PRINT_POP, BC_INST_NQUIT, BC_INST_EXEC_STACK_LEN,
+	BC_INST_SCALE_FUNC, BC_INST_INVALID,
 };
 #endif // DC_ENABLED
 
 #endif // !BC_ENABLE_LIBRARY
 
-#if BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#if BC_ENABLE_EXTRA_MATH
 
+/// A constant for the rand multiplier.
 const BcRandState bc_rand_multiplier = BC_RAND_MULTIPLIER;
 
-#endif // BC_ENABLE_EXTRA_MATH && BC_ENABLE_RAND
+#endif // BC_ENABLE_EXTRA_MATH
 
 #if BC_LONG_BIT >= 64
+
+/// A constant array for the max of a bigdig number as a BcDig array.
 const BcDig bc_num_bigdigMax[] = {
 	709551616U,
 	446744073U,
 	18U,
 };
+
+/// A constant array for the max of 2 times a bigdig number as a BcDig array.
 const BcDig bc_num_bigdigMax2[] = {
 	768211456U,
 	374607431U,
@@ -955,12 +1211,17 @@ const BcDig bc_num_bigdigMax2[] = {
 	282366920U,
 	340U,
 };
+
 #else // BC_LONG_BIT >= 64
+
+/// A constant array for the max of a bigdig number as a BcDig array.
 const BcDig bc_num_bigdigMax[] = {
 	7296U,
 	9496U,
 	42U,
 };
+
+/// A constant array for the max of 2 times a bigdig number as a BcDig array.
 const BcDig bc_num_bigdigMax2[] = {
 	1616U,
 	955U,
@@ -968,13 +1229,19 @@ const BcDig bc_num_bigdigMax2[] = {
 	6744U,
 	1844U,
 };
+
 #endif // BC_LONG_BIT >= 64
 
+/// The size of the bigdig max array.
 const size_t bc_num_bigdigMax_size = sizeof(bc_num_bigdigMax) / sizeof(BcDig);
+
+/// The size of the bigdig max times 2 array.
 const size_t bc_num_bigdigMax2_size = sizeof(bc_num_bigdigMax2) / sizeof(BcDig);
 
+/// A string of digits for easy conversion from characters to digits.
 const char bc_num_hex_digits[] = "0123456789ABCDEF";
 
+/// An array for easy conversion from exponent to power of 10.
 const BcBigDig bc_num_pow10[BC_BASE_DIGS + 1] = {
 	1,
 	10,
@@ -992,6 +1259,8 @@ const BcBigDig bc_num_pow10[BC_BASE_DIGS + 1] = {
 
 #if !BC_ENABLE_LIBRARY
 
+/// An array of functions for binary operators corresponding to the order of
+/// the instructions for the operators.
 const BcNumBinaryOp bc_program_ops[] = {
 	bc_num_pow, bc_num_mul, bc_num_div, bc_num_mod, bc_num_add, bc_num_sub,
 #if BC_ENABLE_EXTRA_MATH
@@ -999,6 +1268,8 @@ const BcNumBinaryOp bc_program_ops[] = {
 #endif // BC_ENABLE_EXTRA_MATH
 };
 
+/// An array of functions for binary operators allocation requests corresponding
+/// to the order of the instructions for the operators.
 const BcNumBinaryOpReq bc_program_opReqs[] = {
 	bc_num_powReq, bc_num_mulReq, bc_num_divReq, bc_num_divReq,
 	bc_num_addReq, bc_num_addReq,
@@ -1007,6 +1278,8 @@ const BcNumBinaryOpReq bc_program_opReqs[] = {
 #endif // BC_ENABLE_EXTRA_MATH
 };
 
+/// An array of unary operator functions corresponding to the order of the
+/// instructions.
 const BcProgramUnary bc_program_unarys[] = {
 	bc_program_negate, bc_program_not,
 #if BC_ENABLE_EXTRA_MATH
@@ -1014,12 +1287,22 @@ const BcProgramUnary bc_program_unarys[] = {
 #endif // BC_ENABLE_EXTRA_MATH
 };
 
+/// A filename for when parsing expressions.
 const char bc_program_exprs_name[] = "<exprs>";
 
+/// A filename for when parsing stdin..
 const char bc_program_stdin_name[] = "<stdin>";
+
+/// A ready message for SIGINT catching.
 const char bc_program_ready_msg[] = "ready for more input\n";
+
+/// The length of the ready message.
 const size_t bc_program_ready_msg_len = sizeof(bc_program_ready_msg) - 1;
+
+/// A list of escape characters that a print statement should treat specially.
 const char bc_program_esc_chars[] = "ab\\efnqrt";
+
+/// A list of characters corresponding to the escape characters above.
 const char bc_program_esc_seqs[] = "\a\b\\\\\f\n\"\r\t";
 
 #endif // !BC_ENABLE_LIBRARY
