@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -45,53 +45,98 @@ bool dc_lex_negCommand(BcLex *l) {
 	return !BC_LEX_NUM_CHAR(c, false, false);
 }
 
+/**
+ * Processes a dc command that needs a register. This is where the
+ * extended-register extension is implemented.
+ * @param l  The lexer.
+ */
 static void dc_lex_register(BcLex *l) {
 
+	// If extended register is enabled and the character is whitespace...
 	if (DC_X && isspace(l->buf[l->i - 1])) {
 
 		char c;
 
+		// Eat the whitespace.
 		bc_lex_whitespace(l);
 		c = l->buf[l->i];
 
-		if (!isalnum(c) && c != '_')
+		// Check for a letter or underscore.
+		if (BC_ERR(!isalpha(c) && c != '_'))
 			bc_lex_verr(l, BC_ERR_PARSE_CHAR, c);
 
+		// Parse a normal identifier.
 		l->i += 1;
 		bc_lex_name(l);
 	}
 	else {
-		bc_vec_npop(&l->str, l->str.len);
+
+		// I don't allow newlines because newlines are used for controlling when
+		// execution happens, and allowing newlines would just be complex.
+		if (BC_ERR(l->buf[l->i - 1] == '\n'))
+			bc_lex_verr(l, BC_ERR_PARSE_CHAR, l->buf[l->i - 1]);
+
+		// Set the lexer string and token.
+		bc_vec_popAll(&l->str);
 		bc_vec_pushByte(&l->str, (uchar) l->buf[l->i - 1]);
 		bc_vec_pushByte(&l->str, '\0');
 		l->t = BC_LEX_NAME;
 	}
 }
 
+/**
+ * Parses a dc string. Since dc's strings need to check for balanced brackets,
+ * we can't just parse bc and dc strings with different start and end
+ * characters. Oh, and dc strings need to check for escaped brackets.
+ * @param l  The lexer.
+ */
 static void dc_lex_string(BcLex *l) {
 
-	size_t depth = 1, nls = 0, i = l->i;
+	size_t depth, nls, i;
 	char c;
+	bool got_more;
 
+	// Set the token and clear the string.
 	l->t = BC_LEX_STR;
-	bc_vec_npop(&l->str, l->str.len);
+	bc_vec_popAll(&l->str);
 
-	for (; (c = l->buf[i]) && depth; ++i) {
+	do {
 
-		if (c == '\\') {
-			c = l->buf[++i];
-			if (!c) break;
-		}
-		else {
-			depth += (c == '[');
-			depth -= (c == ']');
+		depth = 1;
+		nls = 0;
+		got_more = false;
+
+		assert(!l->is_stdin || l->buf == vm.buffer.v);
+
+		// This is the meat. As long as we don't run into the NUL byte, and we
+		// have "depth", which means we haven't completely balanced brackets
+		// yet, we continue eating the string.
+		for (i = l->i; (c = l->buf[i]) && depth; ++i) {
+
+			// Check for escaped brackets and set the depths as appropriate.
+			if (c == '\\') {
+				c = l->buf[++i];
+				if (!c) break;
+			}
+			else {
+				depth += (c == '[');
+				depth -= (c == ']');
+			}
+
+			// We want to adjust the line in the lexer as necessary.
+			nls += (c == '\n');
+
+			if (depth) bc_vec_push(&l->str, &c);
 		}
 
-		nls += (c == '\n');
+		if (BC_ERR(c == '\0' && depth)) {
+			if (!vm.eof && l->is_stdin) got_more = bc_lex_readLine(l);
+			if (got_more) bc_vec_popAll(&l->str);
+		}
 
-		if (depth) bc_vec_push(&l->str, &c);
-	}
+	} while (got_more && depth);
 
+	// Obviously, if we didn't balance, that's an error.
 	if (BC_ERR(c == '\0' && depth)) {
 		l->i = i;
 		bc_lex_err(l, BC_ERR_PARSE_STRING);
@@ -103,25 +148,36 @@ static void dc_lex_string(BcLex *l) {
 	l->line += nls;
 }
 
+/**
+ * Lexes a dc token. This is the dc implementation of BcLexNext.
+ * @param l  The lexer.
+ */
 void dc_lex_token(BcLex *l) {
 
 	char c = l->buf[l->i++], c2;
 	size_t i;
 
+	// If the last token was a command that needs a register, we need to parse a
+	// register, so do so.
 	for (i = 0; i < dc_lex_regs_len; ++i) {
+
+		// If the token is a register token, take care of it and return.
 		if (l->last == dc_lex_regs[i]) {
 			dc_lex_register(l);
 			return;
 		}
 	}
 
+	// These lines are for tokens that easily correspond to one character. We
+	// just set the token.
 	if (c >= '"' && c <= '~' &&
 	    (l->t = dc_lex_tokens[(c - '"')]) != BC_LEX_INVALID)
 	{
 		return;
 	}
 
-	// This is the workhorse of the lexer.
+	// This is the workhorse of the lexer when more complicated things are
+	// needed.
 	switch (c) {
 
 		case '\0':
@@ -136,6 +192,8 @@ void dc_lex_token(BcLex *l) {
 			break;
 		}
 
+		// We don't have the ! command, so we always expect certain things
+		// after the exclamation point.
 		case '!':
 		{
 			c2 = l->buf[l->i];
@@ -146,6 +204,7 @@ void dc_lex_token(BcLex *l) {
 			else bc_lex_invalidChar(l, c);
 
 			l->i += 1;
+
 			break;
 		}
 
@@ -158,9 +217,13 @@ void dc_lex_token(BcLex *l) {
 		case '.':
 		{
 			c2 = l->buf[l->i];
+
+			// If the character after is a number, this dot is part of a number.
+			// Otherwise, it's the BSD dot (equivalent to last).
 			if (BC_NO_ERR(BC_LEX_NUM_CHAR(c2, true, false)))
 				bc_lex_number(l, c);
 			else bc_lex_invalidChar(l, c);
+
 			break;
 		}
 
