@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -53,6 +53,13 @@ inline void bc_parse_pushName(const BcParse *p, char *name, bool var) {
 	bc_parse_pushIndex(p, bc_program_search(p->prog, name, var));
 }
 
+/**
+ * Updates the function, then pushes the instruction and the index. This is a
+ * convenience function.
+ * @param p     The parser.
+ * @param inst  The instruction to push.
+ * @param idx   The index to push.
+ */
 static void bc_parse_update(BcParse *p, uchar inst, size_t idx) {
 	bc_parse_updateFunc(p, p->fidx);
 	bc_parse_push(p, inst);
@@ -61,21 +68,15 @@ static void bc_parse_update(BcParse *p, uchar inst, size_t idx) {
 
 void bc_parse_addString(BcParse *p) {
 
-	BcVec *strs = BC_IS_BC ? &p->func->strs : p->prog->strs;
 	size_t idx;
 
 	BC_SIG_LOCK;
 
-	if (BC_IS_BC) {
-		const char *str = bc_vm_strdup(p->l.str.v);
-		idx = strs->len;
-		bc_vec_push(strs, &str);
-	}
-#if DC_ENABLED
-	else idx = bc_program_insertFunc(p->prog, p->l.str.v) - BC_PROG_REQ_FUNCS;
-#endif // DC_ENABLED
+	idx = bc_program_addString(p->prog, p->l.str.v, p->fidx);
 
-	bc_parse_update(p, BC_INST_STR, idx);
+	// Push the string info.
+	bc_parse_update(p, BC_INST_STR, p->fidx);
+	bc_parse_pushIndex(p, idx);
 
 	BC_SIG_UNLOCK;
 }
@@ -84,26 +85,39 @@ static void bc_parse_addNum(BcParse *p, const char *string) {
 
 	BcVec *consts = &p->func->consts;
 	size_t idx;
-	BcConst c;
+	BcConst *c;
+	BcVec *slabs;
 
+	// Special case 0.
 	if (bc_parse_zero[0] == string[0] && bc_parse_zero[1] == string[1]) {
 		bc_parse_push(p, BC_INST_ZERO);
 		return;
 	}
+
+	// Special case 1.
 	if (bc_parse_one[0] == string[0] && bc_parse_one[1] == string[1]) {
 		bc_parse_push(p, BC_INST_ONE);
 		return;
 	}
 
+	// Get the index.
 	idx = consts->len;
 
 	BC_SIG_LOCK;
 
-	c.val = bc_vm_strdup(string);
-	c.base = BC_NUM_BIGDIG_MAX;
+	// Get the right slab.
+	slabs = p->fidx == BC_PROG_MAIN || p->fidx == BC_PROG_READ ?
+	        &vm.main_const_slab : &vm.other_slabs;
 
-	bc_num_clear(&c.num);
-	bc_vec_push(consts, &c);
+	// Push an empty constant.
+	c = bc_vec_pushEmpty(consts);
+
+	// Set the fields.
+	c->val = bc_slabvec_strdup(slabs, string);
+	c->base = BC_NUM_BIGDIG_MAX;
+
+	// We need this to be able to tell that the number has not been allocated.
+	bc_num_clear(&c->num);
 
 	bc_parse_update(p, BC_INST_NUM, idx);
 
@@ -116,6 +130,8 @@ void bc_parse_number(BcParse *p) {
 	char *exp = strchr(p->l.str.v, 'e');
 	size_t idx = SIZE_MAX;
 
+	// Do we have a number in scientific notation? If so, add a nul byte where
+	// the e is.
 	if (exp != NULL) {
 		idx = ((size_t) (exp - p->l.str.v));
 		*exp = 0;
@@ -125,51 +141,62 @@ void bc_parse_number(BcParse *p) {
 	bc_parse_addNum(p, p->l.str.v);
 
 #if BC_ENABLE_EXTRA_MATH
+	// If we have a number in scientific notation...
 	if (exp != NULL) {
 
 		bool neg;
 
+		// Figure out if the exponent is negative.
 		neg = (*((char*) bc_vec_item(&p->l.str, idx + 1)) == BC_LEX_NEG_CHAR);
 
+		// Add the number and instruction.
 		bc_parse_addNum(p, bc_vec_item(&p->l.str, idx + 1 + neg));
 		bc_parse_push(p, BC_INST_LSHIFT + neg);
 	}
 #endif // BC_ENABLE_EXTRA_MATH
 }
 
-void bc_parse_text(BcParse *p, const char *text) {
+void bc_parse_text(BcParse *p, const char *text, bool is_stdin) {
+
 	// Make sure the pointer isn't invalidated.
 	p->func = bc_vec_item(&p->prog->fns, p->fidx);
-	bc_lex_text(&p->l, text);
+	bc_lex_text(&p->l, text, is_stdin);
 }
 
 void bc_parse_reset(BcParse *p) {
 
 	BC_SIG_ASSERT_LOCKED;
 
+	// Reset the function if it isn't main and switch to main.
 	if (p->fidx != BC_PROG_MAIN) {
 		bc_func_reset(p->func);
 		bc_parse_updateFunc(p, BC_PROG_MAIN);
 	}
 
+	// Reset the lexer.
 	p->l.i = p->l.len;
 	p->l.t = BC_LEX_EOF;
-	p->auto_part = false;
 
 #if BC_ENABLED
 	if (BC_IS_BC) {
+
+		// Get rid of the bc parser state.
+		p->auto_part = false;
 		bc_vec_npop(&p->flags, p->flags.len - 1);
-		bc_vec_npop(&p->exits, p->exits.len);
-		bc_vec_npop(&p->conds, p->conds.len);
-		bc_vec_npop(&p->ops, p->ops.len);
+		bc_vec_popAll(&p->exits);
+		bc_vec_popAll(&p->conds);
+		bc_vec_popAll(&p->ops);
 	}
 #endif // BC_ENABLED
 
+	// Reset the program. This might clear the error.
 	bc_program_reset(p->prog);
 
-	if (BC_ERR(vm.status)) BC_VM_JMP;
+	// Jump if there is an error.
+	if (BC_ERR(vm.status)) BC_JMP;
 }
 
+#ifndef NDEBUG
 void bc_parse_free(BcParse *p) {
 
 	BC_SIG_ASSERT_LOCKED;
@@ -188,6 +215,7 @@ void bc_parse_free(BcParse *p) {
 
 	bc_lex_free(&p->l);
 }
+#endif // NDEBUG
 
 void bc_parse_init(BcParse *p, BcProgram *prog, size_t func) {
 
@@ -201,18 +229,23 @@ void bc_parse_init(BcParse *p, BcProgram *prog, size_t func) {
 
 #if BC_ENABLED
 	if (BC_IS_BC) {
-		bc_vec_init(&p->flags, sizeof(uint16_t), NULL);
+
+		// We always want at least one flag set on the flags stack.
+		bc_vec_init(&p->flags, sizeof(uint16_t), BC_DTOR_NONE);
 		bc_vec_push(&p->flags, &flag);
-		bc_vec_init(&p->exits, sizeof(BcInstPtr), NULL);
-		bc_vec_init(&p->conds, sizeof(size_t), NULL);
-		bc_vec_init(&p->ops, sizeof(BcLexType), NULL);
-		bc_vec_init(&p->buf, sizeof(char), NULL);
+
+		bc_vec_init(&p->exits, sizeof(BcInstPtr), BC_DTOR_NONE);
+		bc_vec_init(&p->conds, sizeof(size_t), BC_DTOR_NONE);
+		bc_vec_init(&p->ops, sizeof(BcLexType), BC_DTOR_NONE);
+		bc_vec_init(&p->buf, sizeof(char), BC_DTOR_NONE);
+
+		p->auto_part = false;
 	}
 #endif // BC_ENABLED
 
 	bc_lex_init(&p->l);
 
+	// Set up the function.
 	p->prog = prog;
-	p->auto_part = false;
 	bc_parse_updateFunc(p, func);
 }
