@@ -2,7 +2,7 @@
 #
 # SPDX-License-Identifier: BSD-2-Clause
 #
-# Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+# Copyright (c) 2018-2021 Gavin D. Howard and contributors.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -32,8 +32,9 @@ set -e
 script="$0"
 testdir=$(dirname "$script")
 
-. "$testdir/../functions.sh"
+. "$testdir/../scripts/functions.sh"
 
+# Command-line processing.
 if [ "$#" -lt 1 ]; then
 	printf 'usage: %s dir [exe [args...]]\n' "$0"
 	printf 'valid dirs are:\n'
@@ -57,47 +58,58 @@ name="$testdir/$d/read.txt"
 results="$testdir/$d/read_results.txt"
 errors="$testdir/$d/read_errors.txt"
 
-out="$testdir/../.log_${d}_test.txt"
+out="$testdir/${d}_outputs/read_results.txt"
+outdir=$(dirname "$out")
+
+# Make sure the directory exists.
+if [ ! -d "$outdir" ]; then
+	mkdir -p "$outdir"
+fi
 
 exebase=$(basename "$exe")
 
+# Set stuff for the correct calculator.
 if [ "$d" = "bc" ]; then
 	options="-lq"
 	halt="halt"
-else
-	options="-x"
-	halt="q"
-fi
-
-if [ "$d" = "bc" ]; then
 	read_call="read()"
 	read_expr="${read_call}\n5+5;"
 else
+	options="-x"
+	halt="q"
 	read_call="?"
 	read_expr="${read_call}"
 fi
 
+# I use these, so unset them to make the tests work.
+unset BC_ENV_ARGS
+unset BC_LINE_LENGTH
+unset DC_ENV_ARGS
+unset DC_LINE_LENGTH
+
 printf 'Running %s read...' "$d"
 
+set +e
+
+# Run read() on every line.
 while read line; do
 
 	printf '%s\n%s\n' "$read_call" "$line" | "$exe" "$@" "$options" > "$out"
-	diff "$results" "$out"
+	checktest "$d" "$?" 'read' "$results" "$out"
 
 done < "$name"
 
 printf 'pass\n'
 
-set +e
-
 printf 'Running %s read errors...' "$d"
 
+# Run read on every line.
 while read line; do
 
 	printf '%s\n%s\n' "$read_call" "$line" | "$exe" "$@" "$options" 2> "$out" > /dev/null
 	err="$?"
 
-	checktest "$d" "$err" "$line" "$out" "$exebase"
+	checkerrtest "$d" "$err" "$line" "$out" "$exebase"
 
 done < "$errors"
 
@@ -110,7 +122,7 @@ read_test=$(printf '%s\n' "$read_call")
 printf '%s\n' "$read_test" | "$exe" "$@" "$opts" 2> "$out" > /dev/null
 err="$?"
 
-checktest "$d" "$err" "$read_test" "$out" "$exebase"
+checkerrtest "$d" "$err" "$read_test" "$out" "$exebase"
 
 printf 'pass\n'
 
@@ -121,6 +133,6 @@ read_test=$(printf '%s' "$read_call")
 printf '%s' "$read_test" | "$exe" "$@" "$opts" 2> "$out" > /dev/null
 err="$?"
 
-checktest "$d" "$err" "$read_test" "$out" "$exebase"
+checkerrtest "$d" "$err" "$read_test" "$out" "$exebase"
 
-printf 'pass\n'
+exec printf 'pass\n'
