@@ -2,7 +2,7 @@
 #
 # SPDX-License-Identifier: BSD-2-Clause
 #
-# Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+# Copyright (c) 2018-2021 Gavin D. Howard and contributors.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -33,6 +33,9 @@ script="$0"
 
 testdir=$(dirname "$script")
 
+. "$testdir/../scripts/functions.sh"
+
+# Command-line processing.
 if [ "$#" -lt 2 ]; then
 	printf 'usage: %s dir test [generate_tests] [time_tests] [exe [args...]]\n' "$0"
 	printf 'valid dirs are:\n'
@@ -71,8 +74,21 @@ else
 	exe="$testdir/../bin/$d"
 fi
 
-out="$testdir/../.log_${d}_test.txt"
+out="$testdir/${d}_outputs/${t}_results.txt"
+outdir=$(dirname "$out")
 
+# Make sure the directory exists.
+if [ ! -d "$outdir" ]; then
+	mkdir -p "$outdir"
+fi
+
+# I use these, so unset them to make the tests work.
+unset BC_ENV_ARGS
+unset BC_LINE_LENGTH
+unset DC_ENV_ARGS
+unset DC_LINE_LENGTH
+
+# Set stuff for the correct calculator.
 if [ "$d" = "bc" ]; then
 	options="-lq"
 	var="BC_LINE_LENGTH"
@@ -83,50 +99,51 @@ else
 	halt="q"
 fi
 
-if [ "${exe#*toybox}" != "$exe" -o "${exe#*busybox}" != "$exe" ]; then
-	if [ "$t" = "print2" -o "$t" = "misc4" ]; then
-		base=$(basename "$exe")
-		printf 'Skipping %s test for %s...\n' "$t" "$base"
-		exit 0
-	fi
-fi
-
+# If the test does not exist...
 if [ ! -f "$name" ]; then
 
+	# Skip if we can't generate.
 	if [ "$generate_tests" -eq 0 ]; then
 		printf 'Skipping %s %s test\n' "$d" "$t"
 		exit 0
 	fi
 
+	# Generate.
 	printf 'Generating %s %s...' "$d" "$t"
-	"$testdir/$d/scripts/$t.$d" > "$name"
+	"$d" "$testdir/$d/scripts/$t.$d" > "$name"
 	printf 'done\n'
 fi
 
+# If the results do not exist, generate..
 if [ ! -f "$results" ]; then
 	printf 'Generating %s %s results...' "$d" "$t"
 	printf '%s\n' "$halt" | "$d" $options "$name" > "$results"
 	printf 'done\n'
 fi
 
+# We set this here because GNU dc does not have it.
 if [ "$d" = "dc" ]; then
 	options="-x"
 fi
 
 export $var=string
 
+set +e
+
 printf 'Running %s %s...' "$d" "$t"
 
 if [ "$time_tests" -ne 0 ]; then
 	printf '\n'
 	printf '%s\n' "$halt" | /usr/bin/time -p "$exe" "$@" $options "$name" > "$out"
+	err="$?"
 	printf '\n'
 else
 	printf '%s\n' "$halt" | "$exe" "$@" $options "$name" > "$out"
+	err="$?"
 fi
 
-diff "$results" "$out"
+checktest "$d" "$err" "$t" "$results" "$out"
 
 rm -f "$out"
 
-printf 'pass\n'
+exec printf 'pass\n'
